@@ -100,7 +100,7 @@ open3_wrap (const char *pathname, int flags, mode_t mode)
 	return open (pathname, flags, mode);
 }
 
-static size_t
+static ssize_t
 read_wrap (int fd, void *buf, size_t count)
 {
 	return read (fd, buf, count);
@@ -112,13 +112,7 @@ close_wrap (int fd)
 	return close (fd);
 }
 
-static int
-fstat_wrap (int filedes, struct stat *buf)
-{
-	return fstat (filedes, buf);
-}
-
-static int
+static ssize_t
 write_wrap (int fd, const void *buf, size_t count)
 {
 	return write (fd, buf, count);
@@ -130,14 +124,37 @@ lseek_wrap (int fildes, off_t offset, int whence)
 	return lseek (fildes, offset, whence);
 }
 
+static int
+isregfile_wrap (int fildes)
+{
+	struct stat st;
+
+	if (fstat (fildes, &st))
+		return 0;
+
+	return S_ISREG(st.st_mode);
+}
+
+static int
+getfilesize_wrap (int fildes, guint32 *size)
+{
+	struct stat st;
+
+	if (fstat (fildes, &st))
+		return -1;
+
+	*size = st.st_size;
+	return 0;
+}
 static MsOleSysWrappers default_wrappers = {
 	open2_wrap,
 	open3_wrap,
 	read_wrap,
 	close_wrap,
-	fstat_wrap,
 	write_wrap,
-	lseek_wrap
+	lseek_wrap,
+	isregfile_wrap,	
+	getfilesize_wrap
 };
 
 static void
@@ -289,7 +306,7 @@ get_block_ptr (MsOle *f, BLP b, gboolean forwrite)
 	g_assert (!attr->data);
 	if (min) {
 		g_assert (min->data);
-#if EXCEL_DEBUG > 0
+#if OLE_DEBUG > 0
 		printf ("Replacing cache block %d with %d\n", min->blk, b);
 #endif
 		if (min->dirty)
@@ -513,11 +530,11 @@ dump_allocation (MsOle *f)
 	printf ("-------------------------------------------------------------\n");
 }
 
-/**
+/*
  * Dump some useful facts.
  * magic: 2       : dump tree
  *        default : dump header and allocation
- **/
+ */
 void
 ms_ole_debug (MsOle *f, int magic)
 {
@@ -687,34 +704,57 @@ extend_file (MsOle *f, guint blocks)
 			f->length+= BB_BLOCK_SIZE;
 		}
 	} else {
-		struct stat st;
 		int file;
 		guint8 *newptr, zero = 0;
+		guint32 filesize;
 		guint32 oldlen;
-		
+		guint32 icount;
+		gchar zeroblock [BB_BLOCK_SIZE];
+
+		memset (zeroblock, zero, BB_BLOCK_SIZE);
 		g_assert (f);
 		file = f->file_des;
 		
 		g_assert (munmap(f->mem, f->length) != -1);
+
 		/* Extend that file by blocks */
-		
-		if ((f->syswrap->fstat(file, &st) == -1) ||
-		    (f->syswrap->lseek (file, st.st_size + BB_BLOCK_SIZE*blocks - 1, SEEK_SET)==(off_t)-1) ||
-		    (f->syswrap->write (file, &zero, 1) == -1)) {
+
+		if (f->syswrap->getfilesize (file, &filesize)) {
+			printf ("Serious error extending file\n");
+			f->mem = 0;
+			return;
+		}
+		if (f->syswrap->lseek (file, 0, SEEK_END) == (off_t)-1) {
+			printf ("Serious error extending file\n");
+			f->mem = 0;
+			return;
+		}
+		for (icount = 0; icount < blocks; icount++) {
+			if (f->syswrap->write (file, zeroblock, BB_BLOCK_SIZE -
+					       ((icount == blocks - 1) ? 1 : 0))
+			    == -1) {
+				printf ("Serious error extending file\n");
+				f->mem = 0;
+				return;
+			}
+		}
+		if (f->syswrap->write (file, &zero, 1) == -1) {
 			printf ("Serious error extending file\n");
 			f->mem = 0;
 			return;
 		}
 
-		oldlen = st.st_size;
-		f->syswrap->fstat(file, &st);
-		f->length = st.st_size;
+		oldlen = filesize;
+
+		if (f->syswrap->getfilesize (file, &(f->length))) {
+			printf ("Warning couldn't get the size of the file\n");
+		}
 		g_assert (f->length == BB_BLOCK_SIZE*blocks + oldlen);
 		if (f->length%BB_BLOCK_SIZE)
 			printf ("Warning file %d non-integer number of blocks\n", f->length);
 		/* NOTE tenix here we don't check if try_mmap is true, because
 		   if it reach here it means f->ole_mmap is true and try_mmap is
-		   true too */
+		   true too. This is related with system wrappers. */
 		newptr = mmap (f->mem, f->length, PROT_READ|PROT_WRITE, MAP_SHARED, file, 0);
 #if OLE_DEBUG > 0
 		if (newptr != f->mem)
@@ -985,13 +1025,13 @@ pps_encode_tree_initial (MsOle *f, GList *list, PPS_IDX *p)
 	g_return_if_fail (list->data);
 	
 	pps = list->data;
+	pps->idx = *p;
+	(*p)++;
 
 #if OLE_DEBUG > 0
 	printf ("encoding '%s' as %d\n", pps->name, pps->idx);
 #endif
 
-	pps->idx = *p;
-	(*p)++;
 	mem = get_pps_ptr (f, pps->idx, TRUE);
 
 	/* Blank stuff I don't understand */
@@ -1031,7 +1071,7 @@ pps_encode_tree_initial (MsOle *f, GList *list, PPS_IDX *p)
 	PPS_SET_PREV (mem, PPS_END_OF_CHAIN);
 	PPS_SET_DIR  (mem, PPS_END_OF_CHAIN);
 
-#if MsOle_DEBUG > 1
+#if OLE_DEBUG > 1
 	printf ("Encode '%s' as \n", pps->name);
 	dump (mem, PPS_BLOCK_SIZE);
 #endif
@@ -1042,10 +1082,10 @@ pps_encode_tree_initial (MsOle *f, GList *list, PPS_IDX *p)
 		pps_encode_tree_initial (f, g_list_next(list), p);
 }
 
-/**
+/*
  * Chain the blocks together afterwards
  * FIXME: Leaks like a sieve
- **/
+ */
 static void
 pps_encode_tree_chain (MsOle *f, GList *list)
 {
@@ -1385,12 +1425,12 @@ ms_ole_unref (MsOle *f)
 
 /**
  * ms_ole_open_vfs:
- * @name: name of OLE2 file to open
- * @try_mmap: TRUE if try to mmap the file to open
- * @wrappers: system functions wrappers, NULL if standard functions are used
+ * @f: FIXME
+ * @name: name of OLE2 file to open.
+ * @try_mmap: TRUE if try to mmap the OLE2 file.
+ * @wrappers: system functions wrappers, NULL if standard functions are used.
  * 
- * Opens a pre-existing OLE2 file, use ms_ole_create ()
- * to create a new file
+ * Opens a pre-existing OLE2 file, use ms_ole_create() to create a new file.
  * 
  * Return value: a handle to the file or NULL on failure.
  **/
@@ -1398,7 +1438,6 @@ MsOleErr
 ms_ole_open_vfs (MsOle **f, const char *name, gboolean try_mmap,
 		 MsOleSysWrappers *wrappers)
 {
-	struct stat st;
 	int prot = PROT_READ | PROT_WRITE;
 	int file;
 
@@ -1419,15 +1458,22 @@ ms_ole_open_vfs (MsOle **f, const char *name, gboolean try_mmap,
 		(*f)->mode = 'r';
 		prot &= ~PROT_WRITE;
 	}
-	if ((file == -1) || (*f)->syswrap->fstat(file, &st)
-	    || !(S_ISREG(st.st_mode))) {
+	if ((file == -1) || !((*f)->syswrap->isregfile (file))) {
+		/* FIXME tenix is not a memory leak not to close file? */
 		printf ("No such file '%s'\n", name);
 		g_free (*f) ;
 		return MS_OLE_ERR_EXIST;
 	}
-	(*f)->length = st.st_size;
+	if ((*f)->syswrap->getfilesize(file, &((*f)->length))) {
+		printf ("Couldn't get the size of file '%s'\n", name);
+		(*f)->syswrap->close (file) ;
+		g_free (*f) ;
+		return MS_OLE_ERR_EXIST;
+	}
 	if ((*f)->length<=0x4c) { /* Bad show */
+#if OLE_DEBUG > 0
 		printf ("File '%s' too short\n", name);
+#endif
 		(*f)->syswrap->close (file) ;
 		g_free (*f) ;
 		return MS_OLE_ERR_FORMAT;
@@ -1437,7 +1483,7 @@ ms_ole_open_vfs (MsOle **f, const char *name, gboolean try_mmap,
 		(*f)->ole_mmap = TRUE;
 		(*f)->mem = mmap (0, (*f)->length, prot, MAP_SHARED, file, 0);
 
-		if (!(*f)->mem || (*f)->mem == MAP_FAILED) {
+		if (!(*f)->mem || (caddr_t)(*f)->mem == (caddr_t)MAP_FAILED) {
 			g_warning ("I can't mmap that file, falling back to slower method");
 			(*f)->ole_mmap = FALSE;
 			(*f)->mem = g_new (guint8, BB_BLOCK_SIZE);
@@ -1494,11 +1540,12 @@ ms_ole_open_vfs (MsOle **f, const char *name, gboolean try_mmap,
 
 /**
  * ms_ole_create_vfs:
- * @name: filename of new OLE file
- * @try_mmap: TRUE if try to mmap the file to open
- * @wrappers: system functions wrappers, NULL if standard functions are used
+ * @f: FIXME
+ * @name: filename of new OLE file.
+ * @try_mmap: TRUE if try to mmap the OLE2 file.
+ * @wrappers: system functions wrappers, NULL if standard functions are used.
  * 
- * Creates an OLE2 file: @name
+ * Creates an OLE2 file: @name.
  *
  * Return value: pointer to new file or NULL on failure.
  **/
@@ -1506,7 +1553,6 @@ MsOleErr
 ms_ole_create_vfs (MsOle **f, const char *name, gboolean try_mmap,
 		   MsOleSysWrappers *wrappers)
 {
-	struct stat st;
 	int file, zero=0;
 	int init_blocks = 1, lp;
 
@@ -1535,9 +1581,11 @@ ms_ole_create_vfs (MsOle **f, const char *name, gboolean try_mmap,
 
 	(*f)->ref_count = 0;
 	(*f)->file_des  = file;
-	(*f)->mode             = 'w';
-	(*f)->syswrap->fstat (file, &st);
-	(*f)->length = st.st_size;
+	(*f)->mode      = 'w';
+	if ((*f)->syswrap->getfilesize (file, &((*f)->length))) {
+		printf ("Warning couldn't get the size of the file '%s'\n",
+			name);
+	}
 	if ((*f)->length % BB_BLOCK_SIZE)
 		printf ("Warning file %d non-integer number of blocks\n",
 			(*f)->length);
@@ -1546,7 +1594,7 @@ ms_ole_create_vfs (MsOle **f, const char *name, gboolean try_mmap,
 		(*f)->ole_mmap = TRUE;
 		(*f)->mem = mmap (0, (*f)->length, PROT_READ|PROT_WRITE,
 				  MAP_SHARED, file, 0);
-		if (!(*f)->mem || (*f)->mem == MAP_FAILED) {
+		if (!(*f)->mem || (caddr_t)(*f)->mem == (caddr_t)MAP_FAILED) {
 			g_warning ("I can't mmap that file, falling back to slower method");
 			(*f)->ole_mmap = FALSE;
 			(*f)->mem  = g_new (guint8, BB_BLOCK_SIZE);
@@ -1622,7 +1670,7 @@ destroy_pps (GList *l)
 
 /**
  * ms_ole_destroy:
- * @f: OLE file handle
+ * @ptr: OLE2 file handle.
  * 
  * Closes @f and truncates any free blocks.
  **/
@@ -1931,11 +1979,11 @@ ms_ole_read_ptr_sb (MsOleStream *s, MsOlePos length)
 	return ans;
 }
 
-/**
+/*
  *  Returns:
  *  0 - on error
  *  1 - on success
- **/
+ */
 static MsOlePos
 ms_ole_read_copy_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 {
@@ -2240,13 +2288,13 @@ ms_ole_write_sb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 /**
  * pps_create:
  * @p: returned pps.
- * @parent: parent pps
- * @name: its name
+ * @parent: parent pps.
+ * @name: its name.
  * @type: the type.
  * 
  * Creates a storage or stream.
  * 
- * Return value: error status
+ * Return value: error status.
  **/
 static MsOleErr
 pps_create (GList **p, GList *parent, const char *name, MsOleType type)
@@ -2479,14 +2527,16 @@ ms_ole_stat (MsOleStat *stat, MsOle *f, const char *path,
 	return MS_OLE_ERR_OK;
 }
 
-/* FIXME tenix next documentation needs update */
+
 /**
  * ms_ole_stream_open:
- * @d: directory entry handle
- * @mode: mode of opening stream
+ * @stream: FIXME
+ * @f: FIXME
+ * @path: FIXME
+ * @fname: FIXME
+ * @mode: mode of opening stream.
  * 
- * Opens the stream with directory handle @d
- * mode is 'r' for read only or 'w' for write only
+ * @mode is 'r' for read only or 'w' for write only.
  * 
  * Return value: handle to an stream.
  **/
@@ -2665,6 +2715,7 @@ ms_ole_stream_duplicate (MsOleStream **s, const MsOleStream * const stream)
  * 
  * Closes stream @s and de-allocates resources.
  * 
+ * Return value: FIXME
  **/
 MsOleErr
 ms_ole_stream_close (MsOleStream **s)
