@@ -19,6 +19,7 @@ typedef struct _ExcelSheet
 	eBiff_version ver;
 	GHashTable *shared_formulae;
 	GList *obj_queue;
+	Range  style_optimize;
 } ExcelSheet;
 
 typedef struct _BiffBoundsheetData
@@ -67,7 +68,6 @@ typedef struct _BiffFontData
 	int script;         /* sub = -1, none = 0, super = 1 */
 	eBiffFontUnderline underline;
 	char *fontname;
-	StyleFont *style_font;
 } BiffFontData;
 
 typedef struct _BiffExternSheetData {
@@ -118,8 +118,7 @@ extern BIFF_BOF_DATA * ms_biff_bof_data_new (BiffQuery * q);
 extern void ms_biff_bof_data_destroy (BIFF_BOF_DATA * data);
 
 extern StyleFormat * biff_format_data_lookup (ExcelWorkbook *wb, guint16 idx);
-extern StyleColor * ms_excel_palette_get (ExcelPalette *pal, guint idx,
-					  StyleColor *contrast);
+extern StyleColor * ms_excel_palette_get (ExcelPalette const *pal, gint idx);
 
 /* A utility routine to handle unexpected BIFF records */
 extern void          ms_excel_unexpected_biff (BiffQuery *q,
