@@ -160,7 +160,7 @@ FormulaFuncData formula_func_data[FORMULA_FUNC_DATA_LEN] =
 /* 89 */	{ "CALLER", -2 },
 /* 90 */	{ "DEREF", -2 },
 /* 91 */	{ "WINDOWS", -2 },
-/* 92 */	{ "SERIESNUM", 4 }, /* Renamed from SERIES */
+/* 92 */	{ "SERIESSUM", 4 }, /* Renamed from SERIES */
 /* 93 */	{ "DOCUMENTS", -2 },
 /* 94 */	{ "ACTIVECELL", -2 },
 /* 95 */	{ "SELECTION", -2 },
@@ -426,7 +426,7 @@ FormulaFuncData formula_func_data[FORMULA_FUNC_DATA_LEN] =
 /* 355 */	{ "OPENDIALOG", -2 },
 /* 356 */	{ "SAVEDIALOG", -2 },
 /* 357 */	{ "VIEWGET", -2 },
-/* 358 */	{ "GETPIVOTDATA", },
+/* 358 */	{ "GETPIVOTDATA", 2 },
 /* 359 */	{ "HYPERLINK", -1 },	/* cell_contents is optional */
 /* 360 */	{ "PHONETIC", -2 },
 /* 361 */	{ "AVERAGEA", -1 },
@@ -869,9 +869,9 @@ ms_excel_parse_formula (ExcelWorkbook *wb, ExcelSheet *sheet, guint8 const *mem,
 		{
 			CellRef *ref=0;
 			if (wb->ver >= eBiffV8) {
-				guint16 extn_idx = MS_OLE_GET_GUINT16(cur) ;
-				ref = getRefV8 (MS_OLE_GET_GUINT16(cur+2),
-						MS_OLE_GET_GUINT16(cur + 4),
+				guint16 extn_idx = MS_OLE_GET_GUINT16 (cur) ;
+				ref = getRefV8 (MS_OLE_GET_GUINT16 (cur + 2),
+						MS_OLE_GET_GUINT16 (cur + 4),
 						fn_col, fn_row, 0) ;
 				make_inter_sheet_ref (wb, extn_idx, ref, 0) ;
 				parse_list_push (&stack, expr_tree_new_var (ref));
@@ -879,11 +879,12 @@ ms_excel_parse_formula (ExcelWorkbook *wb, ExcelSheet *sheet, guint8 const *mem,
 			} else {
 				guint16 extn_idx, first_idx, second_idx;
 
-				ref = getRefV7 (MS_OLE_GET_GUINT8(cur+16), MS_OLE_GET_GUINT16(cur+14),
+				ref = getRefV7 (MS_OLE_GET_GUINT8  (cur + 16),
+						MS_OLE_GET_GUINT16 (cur + 14),
 						fn_col, fn_row, 0) ;
-				extn_idx   = MS_OLE_GET_GUINT16(cur);
-				first_idx  = MS_OLE_GET_GUINT16(cur + 10);
-				second_idx = MS_OLE_GET_GUINT16(cur + 12);
+				extn_idx   = MS_OLE_GET_GUINT16 (cur);
+				first_idx  = MS_OLE_GET_GUINT16 (cur + 10);
+				second_idx = MS_OLE_GET_GUINT16 (cur + 12);
 				make_inter_sheet_ref_v7 (wb, extn_idx, first_idx, second_idx, ref, 0) ;
 				parse_list_push (&stack, expr_tree_new_var (ref));
 				ptg_length = 17 ;
