@@ -318,16 +318,29 @@ expr_tree_array_formula (int const x, int const y, int const rows, int const col
 
 
 static ExprTree *
-expr_tree_array_formula_corner (ExprTree const *expr)
+expr_tree_array_formula_corner (ExprTree const *expr, EvalPosition const *pos)
 {
-	Cell * const corner = expr->u.array.corner.cell;
+	Cell * corner = expr->u.array.corner.cell;
 
-	g_return_val_if_fail (corner, NULL);
+	/* Attempt to set the corner if it is not already set */
+	if (corner == NULL) {
+		g_return_val_if_fail (pos != NULL, NULL);
+		g_return_val_if_fail (pos->sheet != NULL, NULL);
+
+		corner = sheet_cell_get (pos->sheet,
+					 pos->eval_col - expr->u.array.x,
+					 pos->eval_row - expr->u.array.y);
+		((ExprTree *)expr)->u.array.corner.cell = corner;
+	}
+
+	g_return_val_if_fail (corner != NULL, NULL);
 	g_return_val_if_fail (corner->parsed_node != NULL, NULL);
 
 	/* Sanity check incase the corner gets removed for some reason */
 	g_return_val_if_fail (corner->parsed_node != (void *)0xdeadbeef, NULL);
 	g_return_val_if_fail (corner->parsed_node->oper == OPER_ARRAY, NULL);
+	g_return_val_if_fail (corner->parsed_node->u.array.x == 0, NULL);
+	g_return_val_if_fail (corner->parsed_node->u.array.y == 0, NULL);
 
 	return corner->parsed_node;
 }
@@ -531,13 +544,13 @@ eval_funcall (FunctionEvalInfo *s, ExprTree const *tree)
 					else if (a.row == b.row) {
 						int const c = s->pos.eval_col;
 						if (a.col <= c && c <= b.col)
-							v = value_duplicate(value_area_get_x_y (&s->pos, v, c - a.col, 0));
+							v = value_duplicate (value_area_get_x_y (&s->pos, v, c - a.col, 0));
 						else
 							type_mismatch = TRUE;
 					} else if (a.col == b.col) {
 						int const r = s->pos.eval_row;
 						if (a.row <= r && r <= b.row)
-							v = value_duplicate(value_area_get_x_y (&s->pos, v, 0, r - a.row));
+							v = value_duplicate (value_area_get_x_y (&s->pos, v, 0, r - a.row));
 						else
 							type_mismatch = TRUE;
 					} else
@@ -753,13 +766,8 @@ eval_range (FunctionEvalInfo *s, Value *v)
 		for (c = start_col; c <= end_col; ++c) {
 			if ((cell = sheet_cell_get (sheet, c, r)) == NULL)
 				continue;
-			if (cell->generation != gen) {
-				cell->generation = gen;
-				if (cell->parsed_node &&
-				    (cell->flags & CELL_QUEUED_FOR_RECALC))
-					cell_eval (cell);
-			}
-
+			if (cell->generation != gen)
+				cell_eval (cell);
 		}
 }
 
@@ -952,31 +960,31 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 				abort ();
 			}
 		} else {
-			float_t const va = value_get_as_float(a);
-			float_t const vb = value_get_as_float(b);
+			float_t const va = value_get_as_float (a);
+			float_t const vb = value_get_as_float (b);
 			value_release (a);
 			value_release (b);
 
 			switch (tree->oper){
 			case OPER_ADD:
-				return value_new_float(va + vb);
+				return value_new_float (va + vb);
 
 			case OPER_SUB:
-				return value_new_float(va - vb);
+				return value_new_float (va - vb);
 
 			case OPER_MULT:
-				return value_new_float(va * vb);
+				return value_new_float (va * vb);
 
 			case OPER_DIV:
 				return (vb == 0.0)
 				    ? value_new_error (&s->pos,
 						       gnumeric_err_DIV0)
-				    : value_new_float(va / vb);
+				    : value_new_float (va / vb);
 
 			case OPER_EXP:
 				if (va == 0 && vb <= 0)
 					return value_new_error (&s->pos, gnumeric_err_NUM);
-				return value_new_float(pow(va, vb));
+				return value_new_float (pow (va, vb));
 
 			default:
 				break;
@@ -1060,11 +1068,8 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 		if (cell == NULL)
 			return NULL;
 
-		if (cell->generation != s->pos.sheet->workbook->generation){
-			cell->generation = s->pos.sheet->workbook->generation;
-			if (cell->parsed_node && (cell->flags & CELL_QUEUED_FOR_RECALC))
-				cell_eval (cell);
-		}
+		if (cell->generation != s->pos.sheet->workbook->generation)
+			cell_eval (cell);
 
 		return value_duplicate (cell->value);
 	}
@@ -1091,7 +1096,7 @@ eval_expr_real (FunctionEvalInfo *s, ExprTree const *tree)
 			*((Value **)&(tree->u.array.corner.func.value)) = a;
 		} else {
 			ExprTree const * const array =
-			    expr_tree_array_formula_corner (tree);
+			    expr_tree_array_formula_corner (tree, &s->pos);
 			if (array)
 				a = array->u.array.corner.func.value;
 			else
@@ -1139,7 +1144,7 @@ eval_expr (FunctionEvalInfo *s, ExprTree const *tree)
 }
 
 int
-cell_ref_get_abs_col(CellRef const *ref, EvalPosition const *pos)
+cell_ref_get_abs_col (CellRef const *ref, EvalPosition const *pos)
 {
 	g_return_val_if_fail (ref != NULL, 0);
 	g_return_val_if_fail (pos != NULL, 0);
@@ -1151,7 +1156,7 @@ cell_ref_get_abs_col(CellRef const *ref, EvalPosition const *pos)
 }
 
 int
-cell_ref_get_abs_row(CellRef const *ref, EvalPosition const *pos)
+cell_ref_get_abs_row (CellRef const *ref, EvalPosition const *pos)
 {
 	g_return_val_if_fail (ref != NULL, 0);
 	g_return_val_if_fail (pos != NULL, 0);
@@ -1188,7 +1193,7 @@ cell_get_abs_col_row (const CellRef *cell_ref, int eval_col, int eval_row, int *
  * FIXME: strings containing quotes will come out wrong.
  */
 static char *
-do_expr_decode_tree (ExprTree *tree, const ParsePosition *pp,
+do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 		     int paren_level)
 {
 	static struct {
@@ -1316,7 +1321,7 @@ do_expr_decode_tree (ExprTree *tree, const ParsePosition *pp,
 
 	case OPER_VAR: {
 		CellRef *cell_ref = &tree->u.ref;
-		return cellref_name (cell_ref, pp->col, pp->row);
+		return cellref_name (cell_ref, pp);
 	}
 
 	case OPER_CONSTANT: {
@@ -1326,8 +1331,8 @@ do_expr_decode_tree (ExprTree *tree, const ParsePosition *pp,
 		case VALUE_CELLRANGE: {
 			char *a, *b, *res;
 
-			a = cellref_name (&v->v.cell_range.cell_a, pp->col, pp->row);
-			b = cellref_name (&v->v.cell_range.cell_b, pp->col, pp->row);
+			a = cellref_name (&v->v.cell_range.cell_a, pp);
+			b = cellref_name (&v->v.cell_range.cell_b, pp);
 
 			res = g_strconcat (a, ":", b, NULL);
 
@@ -1380,7 +1385,8 @@ do_expr_decode_tree (ExprTree *tree, const ParsePosition *pp,
 		int const y = tree->u.array.y;
 		char *res;
 		if (x != 0 || y != 0) {
-			ExprTree *array = expr_tree_array_formula_corner (tree);
+			ExprTree *array =
+			    expr_tree_array_formula_corner (tree, NULL);
 			if (array) {
 				ParsePosition tmp_pos;
 				tmp_pos.wb  = pp->wb;
@@ -1501,7 +1507,7 @@ cellref_relocate (CellRef * const ref,
 	}
 
 	if (col < 0 || col >= SHEET_MAX_COLS ||
-	    row < 0 || row >= SHEET_MAX_COLS)
+	    row < 0 || row >= SHEET_MAX_ROWS)
 	    return CELLREF_RELOCATE_ERR;
 
 	if (ref->col_relative)
