@@ -207,9 +207,9 @@ append_minute_elapsed (GString *string, int n, const struct tm *time_split, int
 {
 	char *temp = g_alloca (n + 50);
 	int minutes;
-	
+
 	minutes = ((number * 24) + time_split->tm_hour) * 60 + time_split->tm_min;
-	
+
 	sprintf (temp, "%0*d", n, minutes);
 	g_string_append (string, temp);
 
@@ -223,7 +223,7 @@ static void
 append_second (GString *string, int n, const struct tm *time_split)
 {
 	char *temp = g_alloca (n + 4);
-	
+
 	sprintf (temp, "%0*d", n, time_split->tm_sec);
 	g_string_append (string, temp);
 
@@ -231,14 +231,14 @@ append_second (GString *string, int n, const struct tm *time_split)
 }
 
 /*
- * Renders the second field in elapsed 
+ * Renders the second field in elapsed
  */
 static void
 append_second_elapsed (GString *string, int n, const struct tm *time_split, int number)
 {
 	char *temp = g_alloca (n + 50);
 	int seconds;
-	
+
 	seconds = (((number * 24 + time_split->tm_hour) * 60 + time_split->tm_min) * 60) + time_split->tm_sec;
 	sprintf (temp, "%0*d", n, seconds);
 	g_string_append (string, temp);
@@ -335,8 +335,7 @@ format_compile (StyleFormat *format)
 	StyleFormatEntry standard_entries[4];
 	StyleFormatEntry *temp;
 
-	g_list_free (format->format_list);
-	format->format_list = 0;
+	format_destroy (format);
 
 	/* g_string_maybe_expand (string, length); */
 
@@ -399,7 +398,7 @@ format_compile (StyleFormat *format)
 }
 
 static void
-style_entry_free(gpointer data, gpointer user_data)
+style_entry_free (gpointer data, gpointer user_data)
 {
 	StyleFormatEntry *entry = data;
 
@@ -725,26 +724,26 @@ char *
 format_add_thousand (const char *format_string)
 {
 	char *b;
-	
+
 	if (!lc)
 		lc = localeconv ();
 
 	if (strcmp (format_string, "General") == 0){
 		char *s = g_strdup ("#!##0");
 
-		s [1] = THOUSAND_CHAR_OF_LC(lc);
+		s [1] = THOUSAND_CHAR_OF_LC (lc);
 		return s;
 	}
 
-	if (strchr (format_string, THOUSAND_CHAR_OF_LC(lc)) != NULL)
+	if (strchr (format_string, THOUSAND_CHAR_OF_LC (lc)) != NULL)
 		return NULL;
-	
+
 	b = g_malloc (strlen (format_string) + 7);
 	if (!b)
 		return NULL;
 
 	strcpy (b, "#!##0");
-	b [1] = THOUSAND_CHAR_OF_LC(lc);
+	b [1] = THOUSAND_CHAR_OF_LC (lc);
 
 	strcpy (&b[5], format_string);
 
@@ -770,8 +769,8 @@ find_decimal_char (const char *str)
 		switch (*str){
 			/* These ones do not have any argument */
 		case '#': case '?': case '0': case '%':
-		case '-': case '+': case ')': case '': 
-		case ':': case '$': 
+		case '-': case '+': case ')': case '':
+		case ':': case '$':
 		case 'M': case 'm': case 'D': case 'd':
 		case 'Y': case 'y': case 'S': case 's':
 		case '*': case 'h': case 'H': case 'A':
@@ -779,7 +778,7 @@ find_decimal_char (const char *str)
 			break;
 
 			/* Quoted string */
-		case '"': 
+		case '"':
 			for (str++; *str && *str != '"'; str++)
 				;
 			break;
@@ -827,7 +826,7 @@ format_remove_decimal (const char *format_string)
 	 */
 	if (strcmp (format_string, "General") == 0)
 		format_string = "0.0#######";
-	
+
 	ret = g_strdup (format_string);
 	p = (char *) find_decimal_char (ret);
 	if (!p){
@@ -863,12 +862,12 @@ format_add_decimal (const char *format_string)
 
 	if (strcmp (format_string, "General") == 0)
 		format_string = "0";
-		
+
 	p = find_decimal_char (format_string);
 	if (!p){
 		char ret_val [3];
 
-		ret_val [0] = DECIMAL_CHAR_OF_LC(lc);
+		ret_val [0] = DECIMAL_CHAR_OF_LC (lc);
 		ret_val [1] = '0';
 		ret_val [2] = 0;
 		return g_strdup (ret_val);
@@ -919,10 +918,16 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 		switch (c) {
 
 		case '[':
-			if (!ignore_further_elapsed)
+			/* Currency symbol */
+			if (format[1] == '$') {
+				for (format += 2; *format && *format != ']' ; ++format)
+					g_string_append_c (result, *format);
+				if (*format == ']')
+					++format;
+			} else if (!ignore_further_elapsed)
 				time_display_elapsed = TRUE;
 			break;
-			
+
 		case '#':
 			can_render_number = 1;
 			if (info.decimal_separator_seen)
@@ -988,8 +993,9 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 			}
 			/* FIXME: this is a gross hack */
 			{
-				char buffer [40 + DBL_DIG];
-				sprintf (buffer, "%.*g", DBL_DIG, number);
+				char *buffer = g_alloca (40 + info.right_req + 1);
+
+				sprintf (buffer, "%.*e", info.right_req, number);
 
 				g_string_append (result, buffer);
 				goto finish;
@@ -1059,7 +1065,7 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 		case 'M':
 		case 'm': {
 			int n;
-			
+
 			if (!time_split)
 				time_split = split_time (number);
 
@@ -1079,7 +1085,7 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 				append_month (result, n, time_split);
 			break;
 		}
-		
+
 		case 'D':
 		case 'd':
 			if (!time_split)
@@ -1097,7 +1103,7 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 		case 'S':
 		case 's': {
 			int n;
-			
+
 			if (!time_split)
 				time_split = split_time (number);
 
@@ -1113,7 +1119,7 @@ format_number (gdouble number, const StyleFormatEntry *style_format_entry)
 				append_second (result, n, time_split);
 			break;
 		}
-		
+
 		case '*':
 		{
 			/* FIXME FIXME FIXME
@@ -1289,11 +1295,13 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color)
 
 			/*
 			 * Special [h*], [m*], [*s] is using for
+			 * and [$*] are for currencies.
 			 * measuring times, not for specifying colors.
 			 */
 			if (!(first_after_bracket == 'h' ||
 			      first_after_bracket == 's' ||
-			      first_after_bracket == 'm')){
+			      first_after_bracket == 'm' ||
+			      first_after_bracket == '$')){
 				if (color)
 					*color = lookup_color (&entry.format [1], end);
 				entry.format = end+1;
@@ -1365,3 +1373,22 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color)
 
 	return v;
 }
+
+char *
+format_get_thousand ()
+{
+	if (!lc)
+		lc = localeconv ();
+
+	return (lc)->thousands_sep[0] ? (lc)->thousands_sep : ",";
+}
+
+char *
+format_get_decimal ()
+{
+	if (!lc)
+		lc = localeconv ();
+
+	return (lc)->decimal_point[0] ? (lc)->decimal_point : ".";
+}
+
