@@ -18,6 +18,10 @@
 #include "clipboard.h"
 #include "selection.h"
 #include "main.h"
+#include "border.h"
+#include "application.h"
+#include "workbook-cmd-format.h"
+#include "pattern.h"
 
 static GnomeCanvasItem *item_grid_parent_class;
 
@@ -42,31 +46,15 @@ item_grid_destroy (GtkObject *object)
 		(*GTK_OBJECT_CLASS (item_grid_parent_class)->destroy)(object);
 }
 
-struct {
-	gint         width;
-	GdkLineStyle style;
-} static style_border_data[] = {
-	{ 0, GDK_LINE_SOLID },
-	{ 1, GDK_LINE_SOLID },
-	{ 2, GDK_LINE_SOLID },
-	{ 1, GDK_LINE_DOUBLE_DASH },
-	{ 1, GDK_LINE_ON_OFF_DASH },
-	{ 3, GDK_LINE_SOLID },
-	{ 4, GDK_LINE_SOLID },
-	{ 1, GDK_LINE_DOUBLE_DASH },
-	{ 1, GDK_LINE_ON_OFF_DASH },	
-	{ -1, GDK_LINE_SOLID} /* heffalump trap */
-};
-
 static void
 item_grid_realize (GnomeCanvasItem *item)
 {
 	GnomeCanvas *canvas = item->canvas;
 	GdkVisual *visual;
 	GdkWindow *window;
+	GtkStyle  *style;
 	ItemGrid  *item_grid;
 	GdkGC     *gc;
-	gint       i;
 
 	if (GNOME_CANVAS_ITEM_CLASS (item_grid_parent_class)->realize)
 		(*GNOME_CANVAS_ITEM_CLASS (item_grid_parent_class)->realize)(item);
@@ -74,6 +62,14 @@ item_grid_realize (GnomeCanvasItem *item)
 	item_grid = ITEM_GRID (item);
 	window = GTK_WIDGET (item->canvas)->window;
 
+	/* Set the default background color of the canvas itself to white.
+	 * This makes the redraws when the canvas scrolls flicker less.
+	 */
+	style = gtk_style_copy (GTK_WIDGET (item->canvas)->style);
+	style->bg[GTK_STATE_NORMAL] = gs_white;
+	gtk_widget_set_style (GTK_WIDGET (item->canvas), style);
+	gtk_style_unref (style);
+
 	/* Configure the default grid gc */
 	item_grid->grid_gc = gc = gdk_gc_new (window);
 	item_grid->fill_gc = gdk_gc_new (window);
@@ -85,22 +81,6 @@ item_grid_realize (GnomeCanvasItem *item)
 	item_grid->grid_color = gs_light_gray;
 	item_grid->default_color = gs_black;
 
-	for (i = 0; i < BORDER_MAX; i++) {
-		GdkGC *b_gc = gdk_gc_new (window);
-
-		if (style_border_data[i].width < 0)
-			g_warning ("Serious style error");
-
-		gdk_gc_set_line_attributes (b_gc, style_border_data[i].width,
-					    style_border_data[i].style,
-					    GDK_CAP_BUTT, GDK_JOIN_MITER);
-
-		gdk_gc_set_foreground (b_gc, &item_grid->default_color);
-		gdk_gc_set_background (b_gc, &item_grid->background);
-
-		item_grid->border_gc[i] = b_gc;
-	}
-
 	gdk_gc_set_foreground (gc, &item_grid->grid_color);
 	gdk_gc_set_background (gc, &item_grid->background);
 
@@ -126,7 +106,6 @@ static void
 item_grid_unrealize (GnomeCanvasItem *item)
 {
 	ItemGrid *item_grid = ITEM_GRID (item);
-	gint i;
 
 	gdk_gc_unref (item_grid->grid_gc);
 	gdk_gc_unref (item_grid->fill_gc);
@@ -137,9 +116,6 @@ item_grid_unrealize (GnomeCanvasItem *item)
 	item_grid->gc = 0;
 	item_grid->empty_gc = 0;
 
-	for (i = 0; i < BORDER_MAX; i++)
-		gdk_gc_unref (item_grid->border_gc[i]);
-
 	if (GNOME_CANVAS_ITEM_CLASS (item_grid_parent_class)->unrealize)
 		(*GNOME_CANVAS_ITEM_CLASS (item_grid_parent_class)->unrealize)(item);
 }
@@ -248,6 +224,46 @@ item_grid_invert_gc (ItemGrid *item_grid)
 	}
 }
 
+/*
+ * TODO TODO TODO
+ * Correctly support extended cells. Multi-line, or extending to the left
+ * are incorrect currently.
+ */
+static void
+item_grid_draw_border (GdkDrawable *drawable, MStyle *mstyle,
+		       int x, int y, int w, int h,
+		       gboolean const extended_right,
+		       gboolean const extended_left)
+{
+	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_TOP))
+		style_border_draw (drawable,
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_TOP),
+				   x, y, x + w, y);
+	if (!extended_left &&
+	    mstyle_is_element_set (mstyle, MSTYLE_BORDER_LEFT))
+		style_border_draw (drawable, 
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_LEFT),
+				   x, y, x, y + h);
+	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_BOTTOM))
+		style_border_draw (drawable, 
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_BOTTOM),
+				   x, y + h, x + w, y + h);
+	if (!extended_right &&
+	    mstyle_is_element_set (mstyle, MSTYLE_BORDER_RIGHT))
+		style_border_draw (drawable, 
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_RIGHT),
+				   x + w, y, x + w, y + h);
+	
+	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_DIAGONAL))
+		style_border_draw (drawable,
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_DIAGONAL),
+				   x, y + h, x + w, y);
+	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_REV_DIAGONAL))
+		style_border_draw (drawable,
+				   mstyle_get_border (mstyle, MSTYLE_BORDER_REV_DIAGONAL),
+				   x, y, x + w, y + h);
+}
+
 /*
  * Draw a cell.  It gets pixel level coordinates
  *
@@ -256,79 +272,39 @@ item_grid_invert_gc (ItemGrid *item_grid)
 static int
 item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int x1, int y1)
 {
-	GdkGC       *gc     = item_grid->gc;
+	GdkGC      *gc     = item_grid->gc;
+	GdkColor   *col;
 	int         count;
 	int         w, h;
+	MStyle     *mstyle;
+	
+	mstyle = sheet_style_compute (item_grid->sheet, cell->col->pos,
+				      cell->row->pos);
 
 	/* setup foreground */
 	gdk_gc_set_foreground (gc, &item_grid->default_color);
 	if (cell->render_color)
 		gdk_gc_set_foreground (gc, &cell->render_color->color);
-	else {
-		if (cell->style->valid_flags & STYLE_FORE_COLOR)
-			gdk_gc_set_foreground (gc, &cell->style->fore_color->color);
-	}
-
-	if (cell->style->valid_flags & STYLE_BACK_COLOR){
-		gdk_gc_set_background (gc, &cell->style->back_color->color);
+	else if (mstyle_is_element_set (mstyle, MSTYLE_COLOR_FORE)) {
+		col = &mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color;
+		gdk_gc_set_foreground (gc, col);
 	}
 
+	if (mstyle_is_element_set (mstyle, MSTYLE_COLOR_BACK)) {
+		col = &mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color;
+		gdk_gc_set_background (gc, col);
+	} else
+		gdk_gc_set_background (gc, &item_grid->background);
 
 	w = cell->col->pixels;
 	h = cell->row->pixels;
-	if ((cell->style->valid_flags & STYLE_PATTERN) && cell->style->pattern){
-#if 0
-		GnomeCanvasItem *item = GNOME_CANVAS_ITEM (item_grid);
-		int p = cell->style->pattern - 1;
-
-		/*
-		 * Next two lines are commented since the pattern display code of the cell
-		 * have not been tested (written?)
-		 */
-		gdk_gc_set_stipple (gc, GNUMERIC_SHEET (item->canvas)->patterns [p]);
-		gdk_gc_set_fill (gc, GDK_STIPPLED);
-#endif
-		gdk_draw_rectangle (drawable, gc, TRUE,
-				    x1, y1, w, h);
-				    
-		gdk_gc_set_fill (gc, GDK_SOLID);
-		gdk_gc_set_stipple (gc, NULL);
-	}
 
 	/* Draw cell contents BEFORE border */
 	count = cell_draw (cell, item_grid->sheet_view, gc, drawable, x1, y1);
 
-	if ((gnumeric_debugging > 0) &&
-	    (cell->style->valid_flags & STYLE_BORDER) &&
-	    cell->style->border) {
-		StyleBorder     *b = cell->style->border;
-		GdkGC           *gc;
-		StyleBorderType  t;
+	item_grid_draw_border (drawable, mstyle, x1, y1, w, h, count > 1, FALSE);
 
-		t = b->type [STYLE_TOP];
-		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc [t];
-			gdk_draw_line (drawable, gc, x1, y1, x1 + w, y1);
-		}
-
-		t = b->type [STYLE_BOTTOM];
-		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc [t];
-			gdk_draw_line (drawable, gc, x1, y1 + h, x1 + w, y1 + h);
-		}
-
-		t = b->type [STYLE_LEFT];
-		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc [t];
-			gdk_draw_line (drawable, gc, x1, y1, x1, y1 + h);
-		}
-
-		t = b->type [STYLE_RIGHT];
-		if (t != BORDER_NONE) {
-			gc = item_grid->border_gc [t];
-			gdk_draw_line (drawable, gc, x1 + w, y1, x1 + w, y1 + h);
-		}
-	}
+	mstyle_unref (mstyle);
 
 	return count;
 }
@@ -336,34 +312,25 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 static void
 item_grid_paint_empty_cell (GdkDrawable *drawable, ItemGrid *item_grid,
 			    ColRowInfo *ci, ColRowInfo *ri, int col, int row,
-			    int x, int y)
+			    int x, int y,
+			    int const span_count)
 {
-	Style *style;
+	MStyle *mstyle;
+	GdkGC  *gc = item_grid->empty_gc;
 	
-	style = sheet_style_compute (item_grid->sheet, col, row, NULL);
+	mstyle = sheet_style_compute (item_grid->sheet, col, row);
 
-	if (style->valid_flags & (STYLE_PATTERN | STYLE_FORE_COLOR)){
-		/* FIXME: set the GC here */
-	}
+	if (gnumeric_background_set_gc (mstyle, gc, item_grid->canvas_item.canvas))
+		/* Ignore margins. Fill the entire cell (including the right
+		 * hand divider) */
+		gdk_draw_rectangle (drawable, gc, TRUE,
+				    x, y, ci->pixels+1, ri->pixels+1);
 
-	if ((style->valid_flags & STYLE_BACK_COLOR) && (style->back_color)){
-		Sheet *sheet = item_grid->sheet_view->sheet;
-		
-		if (style->back_color == sheet->default_style->back_color){
-			style_destroy (style);
-			return;
-		}
+	item_grid_draw_border (drawable, mstyle, x, y, ci->pixels, ri->pixels,
+			       span_count > 1,
+			       span_count > 0);
 
-		gdk_gc_set_foreground (item_grid->empty_gc, &style->back_color->color);
-		gdk_draw_rectangle (
-			drawable, item_grid->empty_gc, TRUE,
-			x + ci->margin_a, y + ri->margin_b,
-			ci->pixels - ci->margin_b,
-			ri->pixels - ri->margin_b);
-	}
-
-	style_destroy (style);
-	return;
+	mstyle_unref (mstyle);
 }
 
 static void
@@ -378,6 +345,7 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 	int col, row;
 	int x_paint, y_paint, real_x;
 	int diff_x, diff_y;
+	int span_count = 0;
 
 	if (x < 0){
 		g_warning ("x < 0\n");
@@ -447,9 +415,10 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 			if (cell == NULL){
 				item_grid_paint_empty_cell (
 					drawable, item_grid, ci, ri,
-					col, row, x_paint, y_paint);
+					col, row, x_paint, y_paint,
+					--span_count);
 			} else {
-				item_grid_draw_cell (
+				span_count = item_grid_draw_cell (
 					drawable, item_grid, cell,
 					x_paint, y_paint);
 			}
@@ -501,6 +470,8 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 					/*
 					 * Optimization: advance over every
 					 * cell we already painted on
+					 *
+					 * FIXME : This will lose borders.
 					 */
 					end_col = cell->col->pos + count;
 
@@ -636,32 +607,83 @@ context_cell_format_cmd (GtkWidget *widget, Sheet *sheet)
 	context_destroy_menu (widget);
 }
 
+static void
+context_column_width (GtkWidget *widget, Sheet *sheet)
+{
+	workbook_cmd_format_column_width (widget, sheet->workbook);
+	context_destroy_menu (widget);
+}
+
+static void
+context_row_height (GtkWidget *widget, Sheet *sheet)
+{
+	workbook_cmd_format_row_height (widget, sheet->workbook);
+	context_destroy_menu (widget);
+}
+
 typedef enum {
 	IG_ALWAYS,
-	IG_HAVE_SELECTION,
 	IG_SEPARATOR,
+	IG_PASTE,
+	IG_PASTE_SPECIAL,
+	IG_ROW	   = 0x8,
+	IG_COLUMN  = 0x10,
+	IG_DISABLE = 0x20
 } popup_types;
 
 static struct {
-	char         *name;
+	char const * const name;
+	char const * const pixmap;
 	void         (*fn)(GtkWidget *widget, Sheet *item_grid);
-	popup_types  type;
+	popup_types  const type;
 } item_grid_context_menu [] = {
-	{ N_("Cut"),           context_cut_cmd,           IG_ALWAYS         },
-	{ N_("Copy"),          context_copy_cmd,          IG_ALWAYS         },
-	{ N_("Paste"),         context_paste_cmd,         IG_HAVE_SELECTION },
-	{ N_("Paste special"), context_paste_special_cmd, IG_HAVE_SELECTION },
-	{ "",                  NULL,                      IG_SEPARATOR      },
-	{ N_("Insert"),        context_insert_cmd,        IG_ALWAYS  	    },
-	{ N_("Delete"),        context_delete_cmd,        IG_ALWAYS  	    },
-	{ N_("Erase content"), context_clear_cmd,         IG_ALWAYS         },
-	{ "",                  NULL,                      IG_SEPARATOR      },
-	{ N_("Cell format"),   context_cell_format_cmd,   IG_ALWAYS         },
-	{ NULL,                NULL,                      0 }
+	{ N_("Cu_t"),           GNOME_STOCK_PIXMAP_CUT,
+	    &context_cut_cmd,           IG_ALWAYS },
+	{ N_("_Copy"),          GNOME_STOCK_PIXMAP_COPY,
+	    &context_copy_cmd,          IG_ALWAYS },
+	{ N_("_Paste"),         GNOME_STOCK_PIXMAP_PASTE,
+	    &context_paste_cmd,         IG_PASTE },
+	{ N_("Paste _Special"),	NULL,
+	    &context_paste_special_cmd, IG_PASTE_SPECIAL },
+
+	{ "", NULL, NULL, IG_SEPARATOR },
+
+	{ N_("_Insert..."),	NULL,
+	    &context_insert_cmd,        IG_ALWAYS },
+	{ N_("_Delete..."),	NULL,
+	    &context_delete_cmd,        IG_ALWAYS },
+	{ N_("Clear Co_ntents"),NULL,
+	    &context_clear_cmd,         IG_ALWAYS },
+
+	{ "", NULL, NULL, IG_SEPARATOR      },
+
+	{ N_("_Format Cells..."),GNOME_STOCK_PIXMAP_PREFERENCES,
+	    &context_cell_format_cmd,   IG_ALWAYS },
+
+	/* Column specific functions */
+	{ N_("Column _Width..."),NULL,
+	    &context_column_width, IG_COLUMN },
+	{ N_("_Hide"),		 NULL,
+	    NULL,   IG_COLUMN },
+	{ N_("_Unhide"),	 NULL,
+	    NULL,   IG_COLUMN },
+
+	/* Row specific functions (Note some of the labels are duplicated */
+	{ N_("_Row Height..."),	 NULL,
+	    &context_row_height,   IG_ROW },
+	{ N_("_Hide"),		 NULL,
+	    NULL,   IG_ROW },
+	{ N_("_Unhide"),	 NULL,
+	    NULL,   IG_ROW },
+	{ NULL, NULL, NULL, 0 }
 };
 
 static GtkWidget *
-create_popup_menu (Sheet *sheet, int include_paste)
+create_popup_menu (Sheet *sheet,
+		   gboolean const include_paste,
+		   gboolean const include_paste_special,
+		   gboolean const is_col,
+		   gboolean const is_row)
 {
 	GtkWidget *menu, *item;
 	int i;
@@ -669,29 +691,74 @@ create_popup_menu (Sheet *sheet, int include_paste)
 	menu = gtk_menu_new ();
 	item = NULL;
 
-	for (i = 0; item_grid_context_menu [i].name; i++){
-		switch (item_grid_context_menu [i].type){
+	for (i = 0; item_grid_context_menu [i].name; ++i) {
+		popup_types const type = item_grid_context_menu [i].type;
+		char const * const pix_name = item_grid_context_menu [i].pixmap;
+
+		if (type == IG_ROW && !is_row)
+		    continue;
+		if (type == IG_COLUMN && !is_col)
+		    continue;
+
+		switch (item_grid_context_menu [i].type) {
 		case IG_SEPARATOR:
 			item = gtk_menu_item_new ();
 			break;
 
-		case IG_HAVE_SELECTION:
-			if (!include_paste)
-				continue;
-			/* fall down */
-
+		/* Desesitize later */
+		case IG_PASTE : case IG_PASTE_SPECIAL :
+		case IG_ROW :	case IG_COLUMN :
 		case IG_ALWAYS:
-			item = gtk_menu_item_new_with_label (
-				_(item_grid_context_menu [i].name));
+		{
+			/* ICK ! There should be a utility routine for this in gnome or gtk */
+			GtkWidget *label;
+			guint label_accel;
+			label = gtk_accel_label_new ("");
+			label_accel = gtk_label_parse_uline (
+				GTK_LABEL (label),
+				item_grid_context_menu [i].name);
+
+			gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+			gtk_widget_show (label);
+
+			item = gtk_pixmap_menu_item_new	();
+			gtk_container_add (GTK_CONTAINER (item), label);
+
+			if (label_accel != GDK_VoidSymbol) {
+			    if (GTK_IS_MENU (menu))
+				    gtk_widget_add_accelerator (item,
+					    "activate_item",
+					    gtk_menu_ensure_uline_accel_group (GTK_MENU (menu)),
+					    label_accel, 0,
+					    GTK_ACCEL_LOCKED);
+			}
 			break;
+		}
+
 		default:
 			g_warning ("Never reached");
 		}
 
-		gtk_signal_connect (
-			GTK_OBJECT (item), "activate",
-			GTK_SIGNAL_FUNC (item_grid_context_menu [i].fn),
-			sheet);
+		if ((type == IG_PASTE && !include_paste) ||
+		    (type == IG_PASTE_SPECIAL && !include_paste_special) ||
+		    item_grid_context_menu [i].fn == NULL)
+			gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
+
+		if (pix_name != NULL) {
+			GtkWidget *pixmap =
+				gnome_stock_pixmap_widget (GTK_WIDGET (item),
+							   pix_name);
+			gtk_widget_show (pixmap);
+			gtk_pixmap_menu_item_set_pixmap (
+				GTK_PIXMAP_MENU_ITEM (item),
+				pixmap);
+		}
+
+		if (item_grid_context_menu [i].fn)
+			gtk_signal_connect (
+				GTK_OBJECT (item), "activate",
+				GTK_SIGNAL_FUNC (item_grid_context_menu [i].fn),
+				sheet);
 
 		gtk_widget_show (item);
 		gtk_menu_append (GTK_MENU (menu), item);
@@ -701,14 +768,20 @@ create_popup_menu (Sheet *sheet, int include_paste)
 }
 
 void
-item_grid_popup_menu (Sheet *sheet, GdkEvent *event, int col, int row)
+item_grid_popup_menu (Sheet *sheet, GdkEvent *event, int col, int row,
+		       gboolean const is_col,
+		       gboolean const is_row)
 {
 	GtkWidget *menu;
-	int show_paste;
 
-	show_paste = sheet->workbook->clipboard_contents != NULL;
+	/* We can paste if there is something in the clipboard */
+	gboolean const show_paste = !application_clipboard_is_empty ();
+	/* Paste special only applies to copied cells not cut */
+	gboolean const show_paste_special = show_paste &&
+	    (application_clipboard_contents_get () != NULL);
 
-	menu = create_popup_menu (sheet, show_paste);
+	menu = create_popup_menu (sheet, show_paste, show_paste_special,
+				  is_col, is_row);
 
 	gnumeric_popup_menu (GTK_MENU (menu), (GdkEventButton *) event);
 }
@@ -1052,7 +1125,8 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 			
 		case 3:
 			item_grid_popup_menu (item_grid->sheet,
-					      event, col, row);
+					      event, col, row,
+					      FALSE, FALSE);
 			return 1;
 		}
 
@@ -1110,7 +1184,7 @@ item_grid_class_init (ItemGridClass *item_grid_class)
 	GtkObjectClass  *object_class;
 	GnomeCanvasItemClass *item_class;
 
-	item_grid_parent_class = gtk_type_class (gnome_canvas_item_get_type());
+	item_grid_parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
 	object_class = (GtkObjectClass *) item_grid_class;
 	item_class = (GnomeCanvasItemClass *) item_grid_class;
