@@ -22,8 +22,11 @@
 #define CELL_HEIGHT(cell) CELL_DIM(cell,row)
 #define CELL_WIDTH(cell)  CELL_DIM(cell,col)
 
+#if 0
+/* FIXME: need border support. */
 static void
-print_border (GnomePrintContext *pc, double x1, double y1, double x2, double y2, StyleBorder *b, int idx)
+print_border (GnomePrintContext *pc, double x1, double y1, double x2, double y2,
+	      StyleBorder *b, int idx)
 {
 	double width;
 	int dash_mode, dupit;
@@ -73,7 +76,7 @@ print_border (GnomePrintContext *pc, double x1, double y1, double x2, double y2,
 	if (!dupit)
 		return;
 
-	if (x1 == x2){
+	if (x1 == x2) {
 		gnome_print_moveto (pc, x1 + 1, y1);
 		gnome_print_lineto (pc, x2 + 1, y2);
 		gnome_print_stroke (pc);
@@ -84,6 +87,7 @@ print_border (GnomePrintContext *pc, double x1, double y1, double x2, double y2,
 	}
 }
 
+
 static void
 print_cell_border (GnomePrintContext *context, StyleBorder *border,
 		   double x1, double y1, double x2, double y2)
@@ -93,6 +97,7 @@ print_cell_border (GnomePrintContext *context, StyleBorder *border,
 	for (i = 0; i < STYLE_ORIENT_MAX; ++i)
 		print_border (context, x1, y1, x1, y2, border, i);
 }
+#endif
 
 static void
 print_overflow (GnomePrintContext *context, Cell *cell)
@@ -166,10 +171,11 @@ cell_split_text (GnomeFont *font, char const *text, int const width)
 }
 
 static void
-print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double base_y)
+print_cell_text (GnomePrintContext *context, Cell *cell,
+		 double base_x, double base_y, MStyle *mstyle)
 {
-	Style *style = cell->style;
-	GnomeFont *print_font = style->font->font;
+	StyleFont *style_font = mstyle_get_font (mstyle, 1.0);
+	GnomeFont *print_font = style_font->font;
 	double text_width;
 	double font_height;
 	gboolean do_multi_line;
@@ -179,25 +185,30 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 	cell_get_span (cell, &start_col, &end_col);
 	
 	text_width = gnome_font_get_width_string (print_font, cell->text->str);
-	font_height = cell->style->font->size;
+	font_height = style_font->size;
 		
-	if (text_width > cell->col->units && cell_is_number (cell)){
+	if (text_width > cell->col->units && cell_is_number (cell)) {
 		print_overflow (context, cell);
+		style_font_unref (style_font);
 		return;
 	}
 
-	halign = cell_get_horizontal_align (cell);
-	if (halign == HALIGN_JUSTIFY || style->valign == VALIGN_JUSTIFY || style->fit_in_cell)
+	halign = cell_get_horizontal_align (cell,
+					    mstyle_get_align_h (mstyle));
+	if (halign == HALIGN_JUSTIFY ||
+	    mstyle_get_align_v (mstyle) == VALIGN_JUSTIFY ||
+	    mstyle_get_fit_in_cell (mstyle))
 		do_multi_line = TRUE;
 	else
 		do_multi_line = FALSE;
 
-	if (do_multi_line){
+	if (do_multi_line) {
 		GList *lines, *l;
 		int line_count, x_offset, y_offset;
 		double inter_space;
 		
-		lines = cell_split_text (print_font, cell->text->str, cell->col->units);
+		lines = cell_split_text (print_font, cell->text->str,
+					 cell->col->units);
 		line_count = g_list_length (lines);
 		
 		{
@@ -209,23 +220,24 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 			}
 		}
 
-		switch (style->valign){
+		switch (mstyle_get_align_v (mstyle)) {
 		case VALIGN_TOP:
 			y_offset = 0;
 			inter_space = font_height;
 			break;
 			
 		case VALIGN_CENTER:
-			y_offset = - ((cell->row->units - (line_count * font_height))/2);
+			y_offset = - ((cell->row->units -
+				       (line_count * font_height)) / 2);
 			inter_space = font_height;
 			break;
 			
 		case VALIGN_JUSTIFY:
-			if (line_count > 1){
+			if (line_count > 1) {
 				y_offset = 0;
 				inter_space = font_height + 
 					(cell->row->units - (line_count * font_height))
-					/ (line_count-1);
+					/ (line_count - 1);
 				break;
 			} 
 			/* Else, we become a VALIGN_BOTTOM line */
@@ -244,10 +256,10 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 		gnome_print_setfont (context, print_font);
 
 		y_offset -= font_height;
-		for (l = lines; l; l = l->next){
+		for (l = lines; l; l = l->next) {
 			char const * const str = l->data;
 
-			switch (halign){
+			switch (halign) {
 			case HALIGN_LEFT:
 			case HALIGN_JUSTIFY:
 				x_offset = cell->col->margin_a_pt;
@@ -267,7 +279,8 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 				x_offset = cell->col->margin_a_pt;
 			}
 
-			gnome_print_moveto (context, base_x + x_offset, base_y + y_offset);
+			gnome_print_moveto (context, base_x + x_offset,
+					    base_y + y_offset);
 			gnome_print_show (context, str);
 			gnome_print_stroke (context);
 			
@@ -284,21 +297,22 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 		 * compute the pixel difference 
 		 */
 		if (start_col != cell->col->pos)
-			diff = -sheet_col_get_unit_distance (cell->sheet, start_col, cell->col->pos);
+			diff = -sheet_col_get_unit_distance (cell->sheet, start_col,
+							     cell->col->pos);
 		else
 			diff = 0;
 
 		{
 			static int warn_shown;
 
-			if (!warn_shown){
+			if (!warn_shown) {
 				g_warning ("Set clipping");
 				warn_shown = 1;
 			}
 		}
 		
 		len = 0;
-		switch (halign){
+		switch (halign) {
 		case HALIGN_FILL:
 			g_warning ("Unhandled");
 			len = text_width;
@@ -313,7 +327,7 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 			break;
 
 		case HALIGN_CENTER:
-			x = (cell->col->units - text_width)/2;
+			x = (cell->col->units - text_width) / 2;
 			break;
 
 		default:
@@ -325,7 +339,8 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 		gnome_print_setfont (context, print_font);
 		total = 0;
 		do {
-			gnome_print_moveto (context, base_x + x, base_y - cell->row->units);
+			gnome_print_moveto (context, base_x + x,
+					    base_y - cell->row->units);
 			gnome_print_show (context, cell->text->str);
 			gnome_print_stroke (context);
 			
@@ -333,58 +348,80 @@ print_cell_text (GnomePrintContext *context, Cell *cell, double base_x, double b
 			total += len;
 		} while (halign == HALIGN_FILL && total < cell->col->units && len > 0);
 	}
+	style_font_unref (style_font);
 }
 
 static void
-print_cell_background (GnomePrintContext *context, StyleColor *back,
+print_cell_background (GnomePrintContext *context, MStyle *mstyle,
 		       double x1, double y1, double x2, double y2)
 {
-	gnome_print_setrgbcolor (
-		context,
-		back->red   / (double) 0xffff,
-		back->green / (double) 0xffff,
-		back->blue  / (double) 0xffff);
+	int pattern;
 
-	gnome_print_moveto (context, x1, y1);
-	gnome_print_lineto (context, x1, y2);
-	gnome_print_lineto (context, x2, y2);
-	gnome_print_lineto (context, x2, y1);
-	gnome_print_lineto (context, x1, y1);
-	gnome_print_fill (context);
+	g_return_if_fail (mstyle != NULL);
+
+	/*
+	 * Draw the background if the PATTERN is non 0
+	 * Draw a stipple too if the pattern is > 1
+	 */
+	if (!mstyle_is_element_set (mstyle, MSTYLE_PATTERN))
+		return;
+
+	pattern = mstyle_get_pattern (mstyle);
+	if (pattern > 0) {
+		StyleColor *back_col =
+			mstyle_get_color (mstyle, MSTYLE_COLOR_BACK);
+		g_return_if_fail (back_col != NULL);
+
+		gnome_print_setrgbcolor (context,
+					 back_col->red   / (double) 0xffff,
+					 back_col->green / (double) 0xffff,
+					 back_col->blue  / (double) 0xffff);
+
+		/* TODO : Add borders and patterns */
+		/*	print_cell_border (context, style->border, x1, y1, x2, y2); */
+
+		gnome_print_moveto (context, x1, y1);
+		gnome_print_lineto (context, x1, y2);
+		gnome_print_lineto (context, x2, y2);
+		gnome_print_lineto (context, x2, y1);
+		gnome_print_lineto (context, x1, y1);
+		gnome_print_fill (context);
+	}
 }
 
 static void
 print_cell (GnomePrintContext *context, Cell *cell,
 	    double x1, double y1, double x2, double y2)
 {
-	StyleColor *fore = cell->style->fore_color;
+	MStyle     *mstyle = cell_get_mstyle  (cell);
+	StyleColor *fore   = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
 	
 	g_assert (cell != NULL);
 
-	gnome_print_setrgbcolor (context, 0, 0, 0);
-	print_cell_border (context, cell->style->border, x1, y1, x2, y2);
-	print_cell_background (context, cell->style->back_color, x1, y1, x2, y2);
-	
+	print_cell_background (context, mstyle, x1, y1, x2, y2);
+
 	gnome_print_setrgbcolor (context,
-				 fore->red   / (double) 0xfff,
+				 fore->red   / (double) 0xffff,
 				 fore->green / (double) 0xffff,
 				 fore->blue  / (double) 0xffff);
 
 	print_cell_text (context, cell,
 			 x1 + cell->col->margin_a_pt,
-			 y1 + cell->row->margin_b_pt);
+			 y1 + cell->row->margin_b_pt,
+			 mstyle);
+	mstyle_unref (mstyle);
 }
 
 static void
 print_empty_cell (GnomePrintContext *context, Sheet *sheet, int col, int row,
 		  double x1, double y1, double x2, double y2)
 {
-	Style *style;
+	MStyle *mstyle;
 
-	style = sheet_style_compute (sheet, col, row, NULL);
-	print_cell_background (context, style->back_color, x1, y1, x2, y2);
+	mstyle = sheet_style_compute (sheet, col, row);
+	print_cell_background (context, mstyle, x1, y1, x2, y2);
 	
-	style_destroy (style);
+	mstyle_unref (mstyle);
 }
 
 void
@@ -479,19 +516,23 @@ print_cell_grid (GnomePrintContext *context,
 	end_row++;
 	
 	x = base_x;
-	for (col = start_col; col <= end_col; col++){
+	for (col = start_col; col <= end_col; col++) {
 		ColRowInfo *ci = sheet_col_get_info (sheet, col);
 
-		vline (context, x, base_y, base_y - height);
-		x += ci->units + ci->margin_a_pt + ci->margin_b_pt;
+		if (ci) {
+			vline (context, x, base_y, base_y - height);
+			x += ci->units + ci->margin_a_pt + ci->margin_b_pt;
+		}
 	}
 
 	y = base_y;
 	for (row = start_row; row <= end_row; row++){
 		ColRowInfo *ri = sheet_row_get_info (sheet, row);
 
-		hline (context, base_x, base_x + width, y);
-		y -= ri->units + ri->margin_a_pt + ri->margin_b_pt;
+		if (ri) {
+			hline (context, base_x, base_x + width, y);
+			y -= ri->units + ri->margin_a_pt + ri->margin_b_pt;
+		}
 	}
 }
 
