@@ -4,6 +4,35 @@
 #include <gdk/gdk.h>
 #include <libgnomeprint/gnome-font.h>
 
+#define DEFAULT_FONT "Helvetica"
+#define DEFAULT_SIZE 12.0
+
+typedef struct _MStyleBorder      MStyleBorder;
+
+/* Alignment definitions */
+typedef enum {
+	HALIGN_GENERAL =     1,
+	HALIGN_LEFT    =     2,
+	HALIGN_RIGHT   =     4,
+	HALIGN_CENTER  =     8,
+	HALIGN_FILL    =  0x10,
+	HALIGN_JUSTIFY =  0x20
+} StyleHAlignFlags;
+
+typedef enum {
+	VALIGN_TOP     = 1,
+	VALIGN_BOTTOM  = 2,
+	VALIGN_CENTER  = 4,
+	VALIGN_JUSTIFY = 8
+} StyleVAlignFlags;
+
+typedef enum {
+	ORIENT_HORIZ           = 1,
+	ORIENT_VERT_HORIZ_TEXT = 2,
+	ORIENT_VERT_VERT_TEXT  = 4,
+	ORIENT_VERT_VERT_TEXT2 = 8
+} StyleOrientation;
+
 typedef struct {
         char     *format;
 	int      want_am_pm;
@@ -38,106 +67,26 @@ typedef struct {
 	gushort  blue;
 } StyleColor;
 
-/**
- *  The order or the following two records
- * is assumed in xml-io
- **/
-typedef enum {
- 	BORDER_NONE,
- 	BORDER_THIN,
- 	BORDER_MEDIUM,
- 	BORDER_DASHED,
- 	BORDER_DOTTED,
- 	BORDER_THICK,
- 	BORDER_DOUBLE,
- 	BORDER_HAIR,
+/* The order corresponds to the border_buttons name list
+ * in dialog_cell_format_impl */
+typedef enum
+{
+	STYLE_BORDER_TOP,	STYLE_BORDER_BOTTOM,
+	STYLE_BORDER_LEFT,	STYLE_BORDER_RIGHT,
+	STYLE_BORDER_REV_DIAG,STYLE_BORDER_DIAG,
 
- 	BORDER_MAX
-} StyleBorderType;
+	/* These are special.
+	 * They are logical rather than actual borders, however, they
+	 * require extra lines to be drawn so they need to be here.
+	 */
+	STYLE_BORDER_HORIZ, STYLE_BORDER_VERT,
 
-typedef enum {
-	STYLE_TOP    = 0,
- 	STYLE_BOTTOM = 1,
- 	STYLE_LEFT   = 2,
- 	STYLE_RIGHT  = 3,
- 	STYLE_ORIENT_MAX  = 4
-} StyleBorderOrient;
-
-typedef struct {
-	int      ref_count;
-
-	/**
-	 * if the value is BORDER_NONE, then the respective
-	 * color is not allocated, otherwise, it has a
-	 * valid color.
-	 * NB. Use StyleBorderOrient to get orientation
-	 **/
- 	StyleBorderType type[STYLE_ORIENT_MAX];
- 	StyleColor  *color[STYLE_ORIENT_MAX];
-} StyleBorder;
-
-/* Alignment definitions */
-typedef enum {
-	HALIGN_GENERAL =     1,
-	HALIGN_LEFT    =     2,
-	HALIGN_RIGHT   =     4,
-	HALIGN_CENTER  =     8,
-	HALIGN_FILL    =  0x10,
-	HALIGN_JUSTIFY =  0x20
-} StyleHAlignFlags;
-
-typedef enum {
-	VALIGN_TOP     = 1,
-	VALIGN_BOTTOM  = 2,
-	VALIGN_CENTER  = 4,
-	VALIGN_JUSTIFY = 8
-} StyleVAlignFlags;
-
-typedef enum {
-	ORIENT_HORIZ           = 1,
-	ORIENT_VERT_HORIZ_TEXT = 2,
-	ORIENT_VERT_VERT_TEXT  = 4,
-	ORIENT_VERT_VERT_TEXT2 = 8
-} StyleOrientation;
-
-#define STYLE_FORMAT       1
-#define STYLE_FONT         2
-#define STYLE_BORDER       4
-#define STYLE_PATTERN      8
-#define STYLE_ALIGN       16
-#define STYLE_FORE_COLOR  32
-#define STYLE_BACK_COLOR  64
-#define STYLE_MAXIMUM    128
-
-/* Define all of the styles we actually know about */
-#define STYLE_ALL (STYLE_FORMAT | STYLE_FONT | STYLE_BORDER | STYLE_ALIGN | \
-		   STYLE_PATTERN | STYLE_FORE_COLOR | STYLE_BACK_COLOR)
-
-typedef struct {
-	StyleFormat   *format;
-	StyleFont     *font;
-	StyleBorder   *border;
-	StyleColor    *fore_color;
-	StyleColor    *back_color;
-
-	unsigned int pattern:4;
-	unsigned int valign:4;
-	unsigned int halign:6;
-	unsigned int orientation:4;
-	unsigned int fit_in_cell:1;
-	
-	unsigned char valid_flags;
-} Style;
+	STYLE_BORDER_EDGE_MAX
+} StyleBorderLocation;
 
 void           style_init  	      (void);
 void	       style_shutdown         (void);
 
-Style         *style_new   	      (void);
-void           style_merge_to         (Style *target, Style *source);
-Style         *style_duplicate        (const Style *style);
-void           style_destroy          (Style *style);
-Style         *style_new_empty        (void);
-
 StyleFormat   *style_format_new       (const char *name);
 void           style_format_ref       (StyleFormat *sf);
 void           style_format_unref     (StyleFormat *sf);
@@ -156,14 +105,10 @@ void           style_font_ref         (StyleFont *sf);
 void           style_font_unref       (StyleFont *sf);
 
 StyleColor    *style_color_new        (gushort red, gushort green, gushort blue);
-void           style_color_ref        (StyleColor *sc);
+StyleColor    *style_color_ref        (StyleColor *sc);
 void           style_color_unref      (StyleColor *sc);
-
-StyleBorder   *style_border_new_plain (void);
-void           style_border_ref       (StyleBorder *sb);
-void           style_border_unref     (StyleBorder *sb);
-StyleBorder   *style_border_new       (StyleBorderType const border_type[STYLE_ORIENT_MAX],
- 				       StyleColor *border_color[STYLE_ORIENT_MAX]);
+StyleColor    *style_color_black      (void);
+StyleColor    *style_color_white      (void);
 
 /*
  * For hashing Styles
@@ -175,5 +120,6 @@ extern StyleFont *gnumeric_default_font;
 extern StyleFont *gnumeric_default_bold_font;
 extern StyleFont *gnumeric_default_italic_font;
 
-#endif /* GNUMERIC_STYLE_H */
+#include "mstyle.h"
 
+#endif /* GNUMERIC_STYLE_H */
