@@ -6,6 +6,7 @@
 #else
 #   include <gtk/gtkobject.h>
 #endif
+#include <gtk/gtkwidget.h>
 
 #define GNUMERIC_WORKBOOK_GOAD_ID         "IDL:GNOME:Gnumeric:Workbook:1.0"
 #define GNUMERIC_WORKBOOK_FACTORY_GOAD_ID "IDL:GNOME:Gnumeric:WorkbookFactory:1.0"
@@ -31,6 +32,9 @@ typedef struct _PrintInformation PrintInformation;
 struct _WorkbookPrivate;
 typedef struct _WorkbookPrivate WorkbookPrivate;
 
+/* This must be included after the fwd declarations */
+#include "expr.h"
+
 struct _Workbook {
 #ifdef ENABLE_BONOBO
 	/* The base object for the Workbook */
@@ -68,9 +72,6 @@ struct _Workbook {
 	String     *auto_expr_desc;
 	GnomeCanvasItem  *auto_expr_label;
 	
-	/* Styles */
-	Style      style;
-
 	/* The sheets */ 
 	GHashTable *sheets;	/* keeps a list of the Sheets on this workbook */
 
@@ -86,11 +87,6 @@ struct _Workbook {
 
 	int        generation;
 
-	/* The clipboard for this workbook */
-	CellRegion *clipboard_contents;
-
-	gboolean   have_x_selection;
-
 	/* The Symbol table used for naming cell ranges in the workbook */
 	SymbolTable *symbol_names;
 
@@ -102,6 +98,7 @@ struct _Workbook {
 	 * to the asyncronous paste callback
 	 */
 	void       *clipboard_paste_callback_data;
+	gboolean    have_x_selection;
 
 	void       *corba_server;
 
@@ -121,7 +118,7 @@ Workbook   *workbook_new                 (void);
 Workbook   *workbook_core_new            (void);
 Workbook   *workbook_new_with_sheets     (int sheet_count);
 
-void        workbook_set_filename        (Workbook *, const char *);
+gboolean    workbook_set_filename        (Workbook *, const char *);
 void        workbook_set_title           (Workbook *, const char *);
 Workbook   *workbook_try_read            (const char *filename);
 Workbook   *workbook_read                (const char *filename);
@@ -180,12 +177,7 @@ typedef enum {
 	WORKBOOK_FEEDBACK_FONT      = 1 << 3,
 } WorkbookFeedbackType;
 
-void     workbook_feedback_set        (Workbook *,
-				       int feedback_flags,
-				       gboolean italic, gboolean bold,
-				       double size, GnomeFont *font);
-
-extern   Workbook *current_workbook;
+void     workbook_feedback_set        (Workbook *, MStyle *style);
 
 /*
  * Hooks for CORBA bootstrap: they create the 
