@@ -5,6 +5,7 @@
  *   Daniel Veillard <Daniel.Veillard@w3.org>
  *   Miguel de Icaza <miguel@gnu.org>
  *
+ * $Id: xml-io.c,v 1.89 1999/11/18 07:00:19 unammx Exp $
  */
 
 #include <config.h>
@@ -16,6 +17,7 @@
 #include "gnome-xml/tree.h"
 #include "gnome-xml/parser.h"
 #include "color.h"
+#include "border.h"
 #include "sheet-object.h"
 #include "sheet-object-graphic.h"
 #include "print-info.h"
@@ -31,9 +33,7 @@ typedef struct {
 	xmlNodePtr parent;	/* used only for g_hash_table_foreach callbacks */
 	Sheet *sheet;		/* the associated sheet */
 	Workbook *wb;		/* the associated sheet */
-	GHashTable *style_table;/* to generate the styles and then the links to it */
-	int style_count;        /* A style number */
-	xmlNodePtr style_node;  /* The node where we insert the styles */
+	GHashTable *style_table;/* old style styles compatibility */
 } parse_xml_context_t;
 
 /*
@@ -247,6 +247,7 @@ xml_set_gnome_canvas_points (xmlNodePtr node, const char *name,
 {
 	xmlNodePtr child;
 	char *str, *base;
+	char *tstr;
 	int i;
 
 	if (val == NULL)
@@ -272,7 +273,10 @@ xml_set_gnome_canvas_points (xmlNodePtr node, const char *name,
 		}
 		child = child->next;
 	}
-	xmlNewChild (node, NULL, name, xmlEncodeEntities(node->doc, base));
+	xmlNewChild (node, NULL, name,
+		     (tstr = xmlEncodeEntitiesReentrant (node->doc, base)));
+	if (tstr)
+		free (tstr);
 	g_free (base);
 }
 
@@ -423,6 +427,7 @@ xml_set_print_unit (xmlNodePtr node, const char *name,
 {
 	xmlNodePtr  child;
 	char       *txt = "points";
+	char       *tstr;
 
 	if (pu == NULL || name == NULL)
 		return;
@@ -443,10 +448,14 @@ xml_set_print_unit (xmlNodePtr node, const char *name,
 	}
 
 	child = xmlNewChild (node, NULL, "PrintUnit",
-			     xmlEncodeEntities(node->doc, name));
+			     (tstr = xmlEncodeEntitiesReentrant (node->doc, name)));
+	if (tstr)
+		free (tstr);
 	xml_set_value_double (child, "Points", pu->points);
 	xml_set_value (child, "PrefUnit",
-		       xmlEncodeEntities (node->doc, txt));
+		       (tstr = xmlEncodeEntitiesReentrant (node->doc, txt)));
+	if (tstr)
+		free (tstr);
 }
 
 static void
@@ -582,46 +591,50 @@ style_is_default_back (StyleColor *color)
 /*
  * Create an XML subtree of doc equivalent to the given StyleBorder.
  */
-static char *BorderTypes[8] =
-{
-	"none",
- 	"thin",
- 	"medium",
- 	"dashed",
- 	"dotted",
- 	"thick",
- 	"double",
-	"hair"
-};
 
-static char *StyleSideNames[4] =
+static char *StyleSideNames[6] =
 {
  	"Top",
  	"Bottom",
  	"Left",
- 	"Right"
+ 	"Right",
+	"Diagonal",
+	"Rev-Diagonal"
 };
 
 static xmlNodePtr
-xml_write_style_border (parse_xml_context_t *ctxt, StyleBorder *border)
+xml_write_style_border (parse_xml_context_t *ctxt,
+			const MStyle *style)
 {
 	xmlNodePtr cur;
 	xmlNodePtr side;
-	int lp;
+	int        i;
        
-	for (lp = 3; lp >= 0; lp--)
-		if (border->type [lp] != BORDER_NONE)
+	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_REV_DIAGONAL; i++) {
+		MStyleBorder const *border;
+		if (mstyle_is_element_set (style, i) &&
+		    NULL != (border = mstyle_get_border (style, i)) &&
+		    border->line_type != STYLE_BORDER_NONE) {
 			break;
-	if (lp < 0)
+		}
+	}
+	if (i > MSTYLE_BORDER_REV_DIAGONAL)
 		return NULL;
 
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "StyleBorder", NULL);
 	
-	for (lp = 0; lp < 4; lp++){
- 		if (border->type[lp] != BORDER_NONE){
- 			side = xmlNewChild (cur, ctxt->ns, StyleSideNames [lp],
- 					    BorderTypes [border->type [lp]]);
- 			xml_set_color_value (side, "Color", border->color [lp]);
+	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_REV_DIAGONAL; i++) {
+		MStyleBorder const *border;
+		if (mstyle_is_element_set (style, i) &&
+		    NULL != (border = mstyle_get_border (style, i)) &&
+		    border->line_type != STYLE_BORDER_NONE) {
+			StyleBorderType t = border->line_type;
+			StyleColor *col   = border->color;
+ 			side = xmlNewChild (cur, ctxt->ns,
+					    StyleSideNames [i - MSTYLE_BORDER_TOP],
+ 					    NULL);
+ 			xml_set_color_value (side, "Color", col);
+			xml_set_value_int (side, "Style", t);
  		}
 	}
 	return cur;
@@ -630,14 +643,11 @@ xml_write_style_border (parse_xml_context_t *ctxt, StyleBorder *border)
 /*
  * Create a StyleBorder equivalent to the XML subtree of doc.
  */
-static StyleBorder *
-xml_read_style_border (parse_xml_context_t *ctxt, xmlNodePtr tree)
+static void
+xml_read_style_border (parse_xml_context_t *ctxt, xmlNodePtr tree, MStyle *mstyle)
 {
-	StyleBorder *ret;
- 	StyleBorderType style [4] = { BORDER_NONE, BORDER_NONE, BORDER_NONE, BORDER_NONE };
-	StyleColor *color [4] = { NULL, NULL, NULL, NULL };
 	xmlNodePtr side;
-	int lp;
+	int        i;
 
 	if (strcmp (tree->name, "StyleBorder")){
 		fprintf (stderr,
@@ -645,69 +655,89 @@ xml_read_style_border (parse_xml_context_t *ctxt, xmlNodePtr tree)
 			 tree->name);
 	}
 
- 	for (lp = 0; lp < 4; lp++)
- 	{
- 		if ((side = xml_search_child (tree, StyleSideNames [lp])) != NULL)
- 		{
- 			/* FIXME: need to read the proper type */
- 			style [lp] = BORDER_THICK;
- 			xml_get_color_value (side, "Color", &color [lp]);
+	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_REV_DIAGONAL; i++) {
+ 		if ((side = xml_search_child (tree,
+					      StyleSideNames [i - MSTYLE_BORDER_TOP])) != NULL) {
+			int		 t;
+			StyleColor      *color = NULL;
+			MStyleBorder    *border;
+			xml_get_value_int (side, "Style", &t);
+			xml_get_color_value (side, "Color", &color);
+			border = style_border_fetch ((StyleBorderType)t, color, 
+						     style_border_get_orientation (i));
+			if (border)
+				mstyle_set_border (mstyle, i, border);
  		}
 	}
-	
-	ret = style_border_new (style, color);
-
-	return NULL;
 }
 
 /*
  * Create an XML subtree of doc equivalent to the given Style.
  */
 static xmlNodePtr
-xml_write_style (parse_xml_context_t *ctxt, Style *style, int style_idx)
+xml_write_style (parse_xml_context_t *ctxt,
+		 MStyle *style)
 {
-	xmlNodePtr cur, child;
-
-	if ((style->halign == 0) && (style->valign == 0) &&
-	    (style->orientation == 0) && (style->format == NULL) &&
-	    (style->font == NULL) && (style->border == NULL))
-		return NULL;
+	xmlNodePtr  cur, child;
+	char       *tstr;
 
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Style", NULL);
-	if (style_idx != -1)
-		xml_set_value_int (cur, "No", style_idx);
 	
-	xml_set_value_int (cur, "HAlign", style->halign);
-	xml_set_value_int (cur, "VAlign", style->valign);
-	xml_set_value_int (cur, "Fit", style->fit_in_cell);
-	xml_set_value_int (cur, "Orient", style->orientation);
-	xml_set_value_int (cur, "Shade", style->pattern);
+	if (mstyle_is_element_set (style, MSTYLE_ALIGN_H))
+		xml_set_value_int (cur, "HAlign", mstyle_get_align_h (style));
+	if (mstyle_is_element_set (style, MSTYLE_ALIGN_V))
+		xml_set_value_int (cur, "VAlign", mstyle_get_align_v (style));
+	if (mstyle_is_element_set (style, MSTYLE_FIT_IN_CELL))
+		xml_set_value_int (cur, "Fit", mstyle_get_fit_in_cell (style));
+	if (mstyle_is_element_set (style, MSTYLE_ORIENTATION))
+		xml_set_value_int (cur, "Orient", mstyle_get_orientation (style));
+	if (mstyle_is_element_set (style, MSTYLE_PATTERN))
+		xml_set_value_int (cur, "Shade", mstyle_get_pattern (style));
 
-	if (!style_is_default_fore (style->fore_color))
-		xml_set_color_value (cur, "Fore", style->fore_color);
-
-	if (!style_is_default_back (style->back_color))
-		xml_set_color_value (cur, "Back", style->back_color);
-
-	if (style->format != NULL){
-		xml_set_value (cur, "Format", style->format->format);
+	if (mstyle_is_element_set (style, MSTYLE_COLOR_FORE)) {
+		if (!style_is_default_fore (mstyle_get_color (style, MSTYLE_COLOR_FORE)))
+			xml_set_color_value (cur, "Fore", mstyle_get_color (style, MSTYLE_COLOR_FORE));
 	}
+	if (mstyle_is_element_set (style, MSTYLE_COLOR_BACK)) {
+		if (!style_is_default_back (mstyle_get_color (style, MSTYLE_COLOR_BACK)))
+			xml_set_color_value (cur, "Back", mstyle_get_color (style, MSTYLE_COLOR_BACK));
+	}
+	if (mstyle_is_element_set (style, MSTYLE_COLOR_PATTERN)) {
+		if (!style_is_default_back (mstyle_get_color (style, MSTYLE_COLOR_PATTERN)))
+			xml_set_color_value (cur, "PatternColor", mstyle_get_color (style, MSTYLE_COLOR_PATTERN));
+	}
+	if (mstyle_is_element_set (style, MSTYLE_FORMAT))
+		xml_set_value (cur, "Format", mstyle_get_format (style)->format);
+
+	if (mstyle_is_element_set (style, MSTYLE_FONT_NAME) ||
+	    mstyle_is_element_set (style, MSTYLE_FONT_BOLD) ||
+	    mstyle_is_element_set (style, MSTYLE_FONT_ITALIC) ||
+	    mstyle_is_element_set (style, MSTYLE_FONT_SIZE)) {
+		const char *fontname;
+
+		if (mstyle_is_element_set (style, MSTYLE_FONT_NAME))
+			fontname = mstyle_get_font_name (style);
+		else /* backwards compatibility */
+			fontname = "Helvetica";
 
-	if (style->font != NULL){
 		child = xmlNewChild (cur, ctxt->ns, "Font", 
-				     xmlEncodeEntities(ctxt->doc,
-						       style->font->font_name));
-		xml_set_value_double (child, "Unit", style->font->size);
-		xml_set_value_int (child, "Bold", style->font->is_bold);
-		xml_set_value_int (child, "Italic", style->font->is_italic);
-
+				     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, fontname)));
+		if (tstr)
+			free (tstr);
+		if (mstyle_is_element_set (style, MSTYLE_FONT_SIZE))
+			xml_set_value_double (child, "Unit",
+					      mstyle_get_font_size (style));
+		if (mstyle_is_element_set (style, MSTYLE_FONT_BOLD))
+			xml_set_value_int (child, "Bold",
+					   mstyle_get_font_bold (style));
+		if (mstyle_is_element_set (style, MSTYLE_FONT_ITALIC))
+			xml_set_value_int (child, "Italic",
+					   mstyle_get_font_italic (style));
 	}
 
-	if (style->border != NULL){
-		child = xml_write_style_border (ctxt, style->border);
-		if (child)
-			xmlAddChild (cur, child);
-	}
+	child = xml_write_style_border (ctxt, style);
+	if (child)
+		xmlAddChild (cur, child);
 
 	return cur;
 }
@@ -716,6 +746,7 @@ static xmlNodePtr
 xml_write_names (parse_xml_context_t *ctxt, GList *names)
 {
 	xmlNodePtr  cur;
+	char       *tstr;
 
 	if (!names)
 		return NULL;
@@ -731,11 +762,15 @@ xml_write_names (parse_xml_context_t *ctxt, GList *names)
 
 		tmp = xmlNewDocNode (ctxt->doc, ctxt->ns, "Name", NULL);
 		xmlNewChild (tmp, ctxt->ns, "name",
-			     xmlEncodeEntities (ctxt->doc, expr_name->name->str));
+			     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, expr_name->name->str)));
+		if (tstr)
+			free (tstr);
 
 		text = expr_name_value (expr_name);
 		xmlNewChild (tmp, ctxt->ns, "value",
-			     xmlEncodeEntities (ctxt->doc, text));
+			     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, text)));
+		if (tstr)
+			free (tstr);
 		g_free (text);
 
 		xmlAddChild (cur, tmp);
@@ -790,6 +825,7 @@ static xmlNodePtr
 xml_write_summary (parse_xml_context_t *ctxt, SummaryInfo *summary_info)
 {
 	GList *items, *m;
+	char *tstr;
 	xmlNodePtr cur;
 
 	if (!summary_info)
@@ -810,19 +846,25 @@ xml_write_summary (parse_xml_context_t *ctxt, SummaryInfo *summary_info)
 
 			tmp = xmlNewDocNode (ctxt->doc, ctxt->ns, "Item", NULL);
 			xmlNewChild (tmp, ctxt->ns, "name",
-				     xmlEncodeEntities (ctxt->doc, sit->name));
+				     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, sit->name)));
+			if (tstr)
+				free (tstr);
 
 			if (sit->type == SUMMARY_INT) {
 
 				text = g_strdup_printf ("%d", sit->v.i);
 				xmlNewChild (tmp, ctxt->ns, "val-int",
-					     xmlEncodeEntities (ctxt->doc, text));
+					     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, text)));
+				if (tstr)
+					free (tstr);
 
 			} else {
 
 				text = summary_item_as_text (sit);
 				xmlNewChild (tmp, ctxt->ns, "val-string",
-					     xmlEncodeEntities (ctxt->doc, text));
+					     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, text)));
+				if (tstr)
+					free (tstr);
 
 			}
 			g_free (text);
@@ -888,21 +930,28 @@ xml_set_print_hf (xmlNodePtr node, const char *name,
 		  const PrintHF * const hf)
 {
 	xmlNodePtr  child;
+	char *tstr;
 
 	if (hf == NULL || name == NULL)
 		return;
 
 	child = xmlNewChild (node, NULL, name, NULL);
 	xml_set_value (child, "Left",
-		       xmlEncodeEntities (node->doc, hf->left_format));
+		       (tstr = xmlEncodeEntitiesReentrant (node->doc, hf->left_format)));
+	if (tstr)
+		free (tstr);
 	xml_set_value (child, "Middle",
-		       xmlEncodeEntities (node->doc, hf->middle_format));
+		       (tstr = xmlEncodeEntitiesReentrant (node->doc, hf->middle_format)));
+	if (tstr)
+		free (tstr);
 	xml_set_value (child, "Right",
-		       xmlEncodeEntities (node->doc, hf->right_format));
+		       (tstr = xmlEncodeEntitiesReentrant (node->doc, hf->right_format)));
+	if (tstr)
+		free (tstr);
 }
 
 static void
-xml_get_print_hf (xmlNodePtr node, PrintHF * const hf)
+xml_get_print_hf (xmlNodePtr node, PrintHF *const hf)
 {
 	char *txt;
 	
@@ -910,16 +959,31 @@ xml_get_print_hf (xmlNodePtr node, PrintHF * const hf)
 	g_return_if_fail (node != NULL);
 
 	txt = xml_value_get (node, "Left");
-	if (txt)
+	if (txt) {
+		if (hf->left_format)
+			g_free (hf->left_format);
+		
 		hf->left_format = g_strdup (txt);
-
+		free (txt);
+	}
+	
 	txt = xml_value_get (node, "Middle");
-	if (txt)
+	if (txt) {
+		if (hf->middle_format)
+			g_free (hf->middle_format);
+		
 		hf->middle_format = g_strdup (txt);
+		free (txt);
+	}
 
 	txt = xml_value_get (node, "Right");
-	if (txt)
+	if (txt) {
+		if (hf->right_format)
+			g_free (hf->right_format);
+		
 		hf->right_format = g_strdup (txt);
+		free (txt);
+	}
 }
 
 static xmlNodePtr
@@ -1062,7 +1126,7 @@ font_component (const char *fontname, int idx)
 {
 	int i = 0;
 	const char *p = fontname;
-	
+      
 	for (; *p && i < idx; p++){
 		if (*p == '-')
 			i++;
@@ -1074,140 +1138,112 @@ font_component (const char *fontname, int idx)
 }
 
 /**
- * style_font_new_from_x11:
+ * style_font_read_from_x11:
+ * @mstyle: the style to setup to this font.
  * @fontname: an X11-like font name.
- * @scale: scale desired
  *
  * Tries to guess the fontname, the weight and italization parameters
- * to invoke style_font_new
+ * and setup mstyle
  *
  * Returns: A valid style font.
  */
-static StyleFont *
-style_font_new_from_x11 (const char *fontname, double units, double scale)
+static void
+style_font_read_from_x11 (MStyle *mstyle, const char *fontname)
 {
-	StyleFont *sf;
-	char *typeface = "Helvetica";
-	int is_bold = 0;
-	int is_italic = 0;
 	const char *c;
-
+	
 	/*
 	 * FIXME: we should do something about the typeface instead
 	 * of hardcoding it to helvetica.
 	 */
-	
+      
 	c = font_component (fontname, 2);
 	if (strncmp (c, "bold", 4) == 0)
-		is_bold = 1;
+		mstyle_set_font_bold (mstyle, TRUE);
 
 	c = font_component (fontname, 3);
 	if (strncmp (c, "o", 1) == 0)
-		is_italic = 1;
+		mstyle_set_font_italic (mstyle, TRUE);
+
 	if (strncmp (c, "i", 1) == 0)
-		is_italic = 1;
-
-	sf = style_font_new (typeface, units, scale, is_bold, is_italic);
-	if (sf == NULL)
-		sf = style_font_new_from (gnumeric_default_font, scale);
-
-	return sf;
+		mstyle_set_font_italic (mstyle, TRUE);
 }
 
 /*
  * Create a Style equivalent to the XML subtree of doc.
  */
-static Style *
-xml_read_style (parse_xml_context_t *ctxt, xmlNodePtr tree, Style * ret)
+static MStyle *
+xml_read_style (parse_xml_context_t *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child;
 	char *prop;
 	int val;
 	StyleColor *c;
-
-	if (strcmp (tree->name, "Style")){
+	MStyle     *mstyle;
+	
+	mstyle = mstyle_new ();
+	
+	if (strcmp (tree->name, "Style")) {
 		fprintf (stderr,
 			 "xml_read_style: invalid element type %s, 'Style' expected\n",
 			 tree->name);
 	}
-	if (ret == NULL)
-		ret = style_new_empty ();
-	
-	if (ret == NULL)
-		return NULL;
 
-	if (xml_get_value_int (tree, "HAlign", &val)){
-		ret->halign = val;
-		ret->valid_flags |= STYLE_ALIGN;
-	}
-	if (xml_get_value_int (tree, "Fit", &val)){
-		ret->fit_in_cell = val;
-		ret->valid_flags |= STYLE_ALIGN;
-	}
-	if (xml_get_value_int (tree, "VAlign", &val)){
-		ret->valign = val;
-		ret->valid_flags |= STYLE_ALIGN;
-	}
-	if (xml_get_value_int (tree, "Orient", &val)){
-		ret->orientation = val;
-		ret->valid_flags |= STYLE_ALIGN;
-	}
-	if (xml_get_value_int (tree, "Shade", &val)){
-		ret->pattern = val;
-		ret->valid_flags |= STYLE_PATTERN;
-	}
-	if (xml_get_color_value (tree, "Fore", &c)){
-		ret->fore_color = c;
-		ret->valid_flags |= STYLE_FORE_COLOR;
-	}
-	if (xml_get_color_value (tree, "Back", &c)){
-		ret->back_color = c;
-		ret->valid_flags |= STYLE_BACK_COLOR;
-	}
+	if (xml_get_value_int (tree, "HAlign", &val))
+		mstyle_set_align_h (mstyle, val);
+
+	if (xml_get_value_int (tree, "Fit", &val))
+		mstyle_set_fit_in_cell (mstyle, val);
+
+	if (xml_get_value_int (tree, "VAlign", &val))
+		mstyle_set_align_v (mstyle, val);
+
+	if (xml_get_value_int (tree, "Orient", &val))
+		mstyle_set_orientation (mstyle, val);
+
+	if (xml_get_value_int (tree, "Shade", &val))
+		mstyle_set_pattern (mstyle, val);
+
+	if (xml_get_color_value (tree, "Fore", &c))
+		mstyle_set_color (mstyle, MSTYLE_COLOR_FORE, c);
+
+	if (xml_get_color_value (tree, "Back", &c))
+		mstyle_set_color (mstyle, MSTYLE_COLOR_BACK, c);
+
+	if (xml_get_color_value (tree, "PatternColor", &c))
+		mstyle_set_color (mstyle, MSTYLE_COLOR_PATTERN, c);
+
 	prop = xml_value_get (tree, "Format");
-	if (prop != NULL){
-		if (ret->format == NULL){
-			ret->format = style_format_new ((const char *) prop);
-			ret->valid_flags |= STYLE_FORMAT;
-		}
+	if (prop != NULL) {
+		mstyle_set_format (mstyle, prop);
 		free (prop);
 	}
 
 	child = tree->childs;
-	while (child != NULL){
-		if (!strcmp (child->name, "Font")){
+	while (child != NULL) {
+		if (!strcmp (child->name, "Font")) {
 			char *font;
 			double units = 14;
-			int is_bold = 0;
-			int is_italic = 0;
 			int t;
 				
 			xml_get_value_double (child, "Unit", &units);
 
 			if (xml_get_value_int (child, "Bold", &t))
-				is_bold = t;
+				mstyle_set_font_bold (mstyle, t);
+
 			if (xml_get_value_int (child, "Italic", &t))
-				is_italic = t;
+				mstyle_set_font_italic (mstyle, t);
 			
-			font = xmlNodeGetContent(child);
-			if (font != NULL) {
-				StyleFont *sf;
-
-				if (*font == '-'){
-					sf = style_font_new_from_x11 (font, units, 1.0);
-				} else
-					sf = style_font_new (font, units, 1.0, is_bold, is_italic);
-
-				ret->font = sf;
-				free(font);
+			font = xmlNodeGetContent (child);
+			if (font) {
+				if (*font == '-')
+					style_font_read_from_x11 (mstyle, font);
+				else
+					mstyle_set_font_name (mstyle, font);
 			}
-			if (ret->font){
-				ret->valid_flags |= STYLE_FONT;
-			}
-		} else if (!strcmp (child->name, "StyleBorder")){
-			StyleBorder *sb;
-
-			sb = xml_read_style_border (ctxt, child);
+			
+		} else if (!strcmp (child->name, "StyleBorder")) {
+			xml_read_style_border (ctxt, child, mstyle);
 		} else {
 			fprintf (stderr, "xml_read_style: unknown type '%s'\n",
 				 child->name);
@@ -1215,11 +1251,9 @@ xml_read_style (parse_xml_context_t *ctxt, xmlNodePtr tree, Style * ret)
 		child = child->next;
 	}
 
-	/* Now add defaults to any style that was not loaded */
-	return ret;
+	return mstyle;
 }
 
-#if 0
 /*
  * Create an XML subtree of doc equivalent to the given StyleRegion.
  */
@@ -1229,19 +1263,18 @@ xml_write_style_region (parse_xml_context_t *ctxt, StyleRegion *region)
 	xmlNodePtr cur, child;
 
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "StyleRegion", NULL);
-	xml_set_value_int (cur, "startCol", region->range.start_col);
-	xml_set_value_int (cur, "endCol", region->range.end_col);
-	xml_set_value_int (cur, "startRow", region->range.start_row);
-	xml_set_value_int (cur, "endRow", region->range.end_row);
+	xml_set_value_int (cur, "startCol", region->range.start.col);
+	xml_set_value_int (cur, "startRow", region->range.start.row);
+	xml_set_value_int (cur, "endCol",   region->range.end.col);
+	xml_set_value_int (cur, "endRow",   region->range.end.row);
 
-	if (region->style != NULL){
+	if (region->style != NULL) {
 		child = xml_write_style (ctxt, region->style);
 		if (child)
 			xmlAddChild (cur, child);
 	}
 	return cur;
 }
-#endif
 
 /*
  * Create a StyleRegion equivalent to the XML subtree of doc.
@@ -1250,8 +1283,8 @@ static void
 xml_read_style_region (parse_xml_context_t *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child;
-	Style *style = NULL;
-	int start_col = 0, start_row = 0, end_col = 0, end_row = 0;
+	MStyle    *style = NULL;
+	Range      range;
 
 	if (strcmp (tree->name, "StyleRegion")){
 		fprintf (stderr,
@@ -1259,17 +1292,17 @@ xml_read_style_region (parse_xml_context_t *ctxt, xmlNodePtr tree)
 			 tree->name);
 		return;
 	}
-	xml_get_value_int (tree, "startCol", &start_col);
-	xml_get_value_int (tree, "startRow", &start_row);
-	xml_get_value_int (tree, "endCol", &end_col);
-	xml_get_value_int (tree, "endRow", &end_row);
+	xml_get_value_int (tree, "startCol", &range.start.col);
+	xml_get_value_int (tree, "startRow", &range.start.row);
+	xml_get_value_int (tree, "endCol",   &range.end.col);
+	xml_get_value_int (tree, "endRow",   &range.end.row);
 	child = tree->childs;
-	if (child != NULL)
-		style = xml_read_style (ctxt, child, NULL);
-	if (style != NULL)
-		sheet_style_attach (ctxt->sheet, start_col, start_row, end_col,
-				    end_row, style);
 
+	if (child)
+		style = xml_read_style (ctxt, child);
+
+	if (style)
+		sheet_style_attach (ctxt->sheet, range, style);
 }
 
 /*
@@ -1352,29 +1385,40 @@ xml_read_colrow_info (parse_xml_context_t *ctxt, xmlNodePtr tree, ColRowInfo *re
 static xmlNodePtr
 xml_write_sheet_object (parse_xml_context_t *ctxt, SheetObject *object)
 {
-	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (object);
+	SheetObjectGraphic *sog;
 	xmlNodePtr cur = NULL;
 
-	switch (sog->type){
-	case SHEET_OBJECT_BOX:{
-			SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
-
-			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Rectangle", NULL);
-			if (sof->fill_color != NULL)
-				xml_set_value_string (cur, "FillColor", sof->fill_color);
-			xml_set_value_int (cur, "Pattern", sof->pattern);
-			break;
+	if (!IS_SHEET_GRAPHIC_OBJECT (object)) {
+		static gboolean warned = FALSE;
+		if (!warned) {
+			g_warning ("Discarding non-graphic embedded objects");
+			warned = TRUE;
 		}
+		return NULL;
+	}
 
-	case SHEET_OBJECT_OVAL:{
-			SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
+	sog = SHEET_OBJECT_GRAPHIC (object);
 
-			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Ellipse", NULL);
-			if (sof->fill_color != NULL)
-				xml_set_value_string (cur, "FillColor", sof->fill_color);
-			xml_set_value_int (cur, "Pattern", sof->pattern);
-			break;
-		}
+	switch (sog->type) {
+	case SHEET_OBJECT_BOX: {
+		SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
+		
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Rectangle", NULL);
+		if (sof->fill_color != NULL)
+			xml_set_value_string (cur, "FillColor", sof->fill_color);
+		xml_set_value_int (cur, "Pattern", sof->pattern);
+		break;
+	}
+
+	case SHEET_OBJECT_OVAL: {
+		SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
+		
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Ellipse", NULL);
+		if (sof->fill_color != NULL)
+			xml_set_value_string (cur, "FillColor", sof->fill_color);
+		xml_set_value_int (cur, "Pattern", sof->pattern);
+		break;
+	}
 
 	case SHEET_OBJECT_ARROW:
 		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Arrow", NULL);
@@ -1387,7 +1431,7 @@ xml_write_sheet_object (parse_xml_context_t *ctxt, SheetObject *object)
 	default :
 		cur = NULL;
 	}
-	if (cur == NULL)
+	if (!cur)
 		return NULL;
 	
 	xml_set_gnome_canvas_points (cur, "Points", object->bbox_points);
@@ -1458,28 +1502,28 @@ xml_write_cell (parse_xml_context_t *ctxt, Cell *cell)
 {
 	xmlNodePtr cur;
 	char *text;
-	int style_id;
+	char *tstr;
 
-	style_id = GPOINTER_TO_INT (g_hash_table_lookup (ctxt->style_table, cell->style));
-	g_assert (style_id != 0);
-	
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Cell", NULL);
 	xml_set_value_int (cur, "Col", cell->col->pos);
 	xml_set_value_int (cur, "Row", cell->row->pos);
-	xml_set_value_int (cur, "Style", style_id);
+	xml_set_value_int (cur, "Style", 0); /* Backwards compatible */
 
 	text = cell_get_content (cell);
 	xmlNewChild (cur, ctxt->ns, "Content",
-		     xmlEncodeEntities(ctxt->doc, text));
+		     (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, text)));
+	if (tstr)
+		free (tstr);
 	g_free (text);
 
  	text = cell_get_comment(cell);
  	if (text) {
 		xmlNewChild(cur, ctxt->ns, "Comment", 
-			    xmlEncodeEntities(ctxt->doc, text));
+			    (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, text)));
+	if (tstr)
+		free (tstr);
 		g_free(text);
  	}
-	
 
 	return cur;
 }
@@ -1490,16 +1534,15 @@ xml_write_cell (parse_xml_context_t *ctxt, Cell *cell)
 static Cell *
 xml_read_cell (parse_xml_context_t *ctxt, xmlNodePtr tree)
 {
-	Style *style;
 	Cell *ret;
 	xmlNodePtr childs;
 	int row = 0, col = 0;
 	char *content = NULL;
 	char *comment = NULL;
+	int  style_idx;
 	gboolean style_read = FALSE;
-	int style_idx;
 	
-	if (strcmp (tree->name, "Cell")){
+	if (strcmp (tree->name, "Cell")) {
 		fprintf (stderr,
 		 "xml_read_cell: invalid element type %s, 'Cell' expected`\n",
 			 tree->name);
@@ -1520,65 +1563,61 @@ xml_read_cell (parse_xml_context_t *ctxt, xmlNodePtr tree)
 		return NULL;
 	}
 
+
 	/*
-	 * New file format includes an index pointer to the Style
+	 * This style code is a gross anachronism that slugs performance
+	 * in the common case this data won't exist. In the long term all
+	 * files will make the 0.41 - 0.42 transition and this can go.
+	 * Newer file format includes an index pointer to the Style
 	 * Old format includes the Style online
 	 */
-	if (xml_get_value_int (tree, "Style", &style_idx)){
-		Style *s;
+	if (xml_get_value_int (tree, "Style", &style_idx)) {
+		MStyle *mstyle;
 		
 		style_read = TRUE;
-		s = g_hash_table_lookup (ctxt->style_table, GINT_TO_POINTER (style_idx));
-		if (s) {
-			Style *copy;
-
-			/*
-			 * The main style is the style we read, but this
-			 * style might be incomplete (ie, older formats might
-			 * not have full styles.
-			 *
-			 * so we merge the missing bits from the current cell
-			 * style
-			 */
-			copy = style_duplicate (s);
-			style_merge_to (copy, ret->style);
-			style_destroy (ret->style);
-			ret->style = copy;
-		} else {
-			printf ("Error: could not find style %d\n", style_idx);
-		}
+		mstyle = g_hash_table_lookup (ctxt->style_table,
+					      GINT_TO_POINTER (style_idx));
+		if (mstyle) {
+			mstyle_ref (mstyle);
+			sheet_style_attach_single (ctxt->sheet, col, row,
+						   mstyle);
+		} /* else reading a newer version with style_idx == 0 */
 	}
 	
 	childs = tree->childs;
 	while (childs != NULL) {
+		/*
+		 * This style code is a gross anachronism that slugs performance
+		 * in the common case this data won't exist. In the long term all
+		 * files will make the 0.41 - 0.42 transition and this can go.
+		 * This is even older backwards compatibility than 0.41 - 0.42
+		 */
 		if (!strcmp (childs->name, "Style")) {
-			if (!style_read){
-				style = xml_read_style (ctxt, childs, NULL);
-				if (style){
-					style_merge_to (style, ret->style);
-					style_destroy (ret->style);
-					ret->style = style;
-				}
+			if (!style_read) {
+				MStyle *mstyle;
+				mstyle = xml_read_style (ctxt, childs);
+				if (mstyle)
+					sheet_style_attach_single (ctxt->sheet, col, row,
+								   mstyle);
 			}
 		}
 		if (!strcmp (childs->name, "Content"))
-			content = xmlNodeGetContent(childs);
+			content = xmlNodeGetContent (childs);
 		if (!strcmp (childs->name, "Comment")) {
-			comment = xmlNodeGetContent(childs);
+			comment = xmlNodeGetContent (childs);
  			if (comment) {
- 				cell_set_comment(ret, comment);
- 				free(comment);
+ 				cell_set_comment (ret, comment);
+ 				free (comment);
 			}
  		}
 		childs = childs->next;
 	}
-	
 	if (content == NULL)
-		content = xmlNodeGetContent(tree);
+		content = xmlNodeGetContent (tree);
 	if (content != NULL) {
 		char *p = content + strlen (content);
 
-		while (p > content){
+		while (p > content) {
 			p--;
 			if (*p != ' ' && *p != '\n')
 				break;
@@ -1591,7 +1630,7 @@ xml_read_cell (parse_xml_context_t *ctxt, xmlNodePtr tree)
 		 */
 		if (ret->parsed_node == NULL ||
 		    OPER_ARRAY != ret->parsed_node->oper)
-		cell_set_text_simple (ret, content);
+			cell_set_text_simple (ret, content);
 		free (content);
 	} else
 		cell_set_text_simple (ret, "");
@@ -1615,39 +1654,22 @@ xml_write_cell_to (gpointer key, gpointer value, gpointer data)
 	xmlAddChild (ctxt->parent, cur);
 }
 
-static void
-add_style (gpointer key, gpointer value, gpointer data)
+static xmlNodePtr
+xml_write_styles (parse_xml_context_t *ctxt, GList *l)
 {
-	parse_xml_context_t *ctxt = data;
-	Cell *cell = (Cell *) value;
-	xmlNodePtr child;
+	xmlNodePtr cur;
+
+	if (!l)
+		return NULL;
+
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Styles", NULL);
+	while (l && l->next) {
+		StyleRegion *sr = l->data;
+		xmlAddChild (cur, xml_write_style_region (ctxt, sr));
+		l = g_list_next (l);
+	}
 	
-	if (g_hash_table_lookup (ctxt->style_table, cell->style))
-		return;
-
-	child = xml_write_style (ctxt, cell->style, ctxt->style_count);
-	xmlAddChild (ctxt->style_node, child);
-		     
-	g_hash_table_insert (ctxt->style_table, cell->style, GINT_TO_POINTER (ctxt->style_count));
-	ctxt->style_count++;
-	
-}
-
-static void
-xml_cell_styles_init (parse_xml_context_t *ctxt, xmlNodePtr cur, Sheet *sheet)
-{
-	ctxt->style_node = xmlNewChild (cur, ctxt->ns, "CellStyles", NULL);
-
-	ctxt->style_table = g_hash_table_new (style_hash, style_compare);
-	ctxt->style_count = 1;
-
-	g_hash_table_foreach (sheet->cell_hash, add_style, ctxt);
-}
-
-static void
-xml_cell_styles_shutdown (parse_xml_context_t *ctxt)
-{
-	g_hash_table_destroy (ctxt->style_table);
+	return cur;
 }
 
 /*
@@ -1663,7 +1685,11 @@ xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 	xmlNodePtr cells;
 	xmlNodePtr objects;
 	xmlNodePtr printinfo;
-	char str[50];
+	xmlNodePtr styles;
+	GList     *style_regions;
+
+	char str [50];
+	char *tstr;
 
 	/*
 	 * General informations about the Sheet.
@@ -1672,7 +1698,9 @@ xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 	if (cur == NULL)
 		return NULL;
 	xmlNewChild (cur, ctxt->ns, "Name", 
-	             xmlEncodeEntities(ctxt->doc, sheet->name));
+	             (tstr = xmlEncodeEntitiesReentrant (ctxt->doc, sheet->name)));
+	if (tstr)
+		free (tstr);
 	sprintf (str, "%d", sheet->cols.max_used);
 	xmlNewChild (cur, ctxt->ns, "MaxCol", str);
 	sprintf (str, "%d", sheet->rows.max_used);
@@ -1690,12 +1718,17 @@ xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 	printinfo = xml_write_print_info (ctxt, sheet->print_info);
 	if (printinfo)
 		xmlAddChild (cur, printinfo);
-	
+
 	/*
-	 * Styles used by the cells on this sheet
+	 * Styles
 	 */
-	xml_cell_styles_init (ctxt, cur, sheet);
-	
+	style_regions = sheet_get_style_list (sheet);
+	style_regions = g_list_reverse (style_regions);
+	styles = xml_write_styles (ctxt, style_regions);
+	g_list_free (style_regions);
+	if (styles)
+		xmlAddChild (cur, styles);
+
 	/*
 	 * Cols informations.
 	 */
@@ -1750,51 +1783,10 @@ xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 	cells = xmlNewChild (cur, ctxt->ns, "Cells", NULL);
 	ctxt->parent = cells;
 	g_hash_table_foreach (sheet->cell_hash, xml_write_cell_to, ctxt);
-	sheet->modified = 0;
 
-	xml_cell_styles_shutdown (ctxt);
-	
 	return cur;
 }
 
-static void
-xml_read_cell_styles (parse_xml_context_t *ctxt, xmlNodePtr tree)
-{
-	xmlNodePtr styles, child;
-
-	ctxt->style_table = g_hash_table_new (g_direct_hash, g_direct_equal);
-	
-	child = xml_search_child (tree, "CellStyles");
-	if (child == NULL)
-		return;
-	
-	for (styles = child->childs; styles; styles = styles->next){
-		Style *s;
-		int style_idx;
-		
-		if (xml_get_value_int (styles, "No", &style_idx)){
-			s = xml_read_style (ctxt, styles, NULL);
-			g_hash_table_insert (
-				ctxt->style_table,
-				GINT_TO_POINTER (style_idx),
-				s);
-		}
-	}
-}
-
-static void
-destroy_style (gpointer key, gpointer value, gpointer data)
-{
-	style_destroy (value);
-}
-
-static void
-xml_dispose_read_cell_styles (parse_xml_context_t *ctxt)
-{
-	g_hash_table_foreach (ctxt->style_table, destroy_style, NULL);
-	g_hash_table_destroy (ctxt->style_table);
-}
-
 static void
 xml_read_styles (parse_xml_context_t *ctxt, xmlNodePtr tree)
 {
@@ -1851,6 +1843,43 @@ xml_read_rows_info (parse_xml_context_t *ctxt, Sheet *sheet, xmlNodePtr tree)
 	}
 }
 
+static void
+xml_read_cell_styles (parse_xml_context_t *ctxt, xmlNodePtr tree)
+{
+	xmlNodePtr styles, child;
+
+	ctxt->style_table = g_hash_table_new (g_direct_hash, g_direct_equal);
+	
+	child = xml_search_child (tree, "CellStyles");
+	if (child == NULL)
+		return;
+	
+	for (styles = child->childs; styles; styles = styles->next) {
+		MStyle *mstyle;
+		int style_idx;
+		
+		if (xml_get_value_int (styles, "No", &style_idx)) {
+			mstyle = xml_read_style (ctxt, styles);
+			g_hash_table_insert (
+				ctxt->style_table,
+				GINT_TO_POINTER (style_idx),
+				mstyle);
+		}
+	}
+}
+static void
+destroy_style (gpointer key, gpointer value, gpointer data)
+{
+	mstyle_unref (value);
+}
+
+static void
+xml_dispose_read_cell_styles (parse_xml_context_t *ctxt)
+{
+	g_hash_table_foreach (ctxt->style_table, destroy_style, NULL);
+	g_hash_table_destroy (ctxt->style_table);
+}
+
 /*
  * Create a Sheet equivalent to the XML subtree of doc.
  */
@@ -1955,9 +1984,9 @@ xml_workbook_write (parse_xml_context_t *ctxt, Workbook *wb)
 	if (child)
 		xmlAddChild (cur, child);
 
-	child = xml_write_style (ctxt, &wb->style, -1);
+/*	child = xml_write_style (ctxt, &wb->style, -1);
 	if (child)
-		xmlAddChild (cur, child);
+	xmlAddChild (cur, child);*/
 
 	child = xmlNewDocNode (ctxt->doc, ctxt->ns, "Geometry", NULL);
 	xml_set_value_int (child, "Width", wb->toplevel->allocation.width);
@@ -2048,9 +2077,9 @@ xml_workbook_read (Workbook *wb, parse_xml_context_t *ctxt, xmlNodePtr tree)
 /*      gtk_widget_set_usize(wb->toplevel, width, height); */
 	}
 
-	child = xml_search_child (tree, "Style");
+/*	child = xml_search_child (tree, "Style");
 	if (child != NULL)
-		xml_read_style (ctxt, child, &wb->style);
+	xml_read_style (ctxt, child, &wb->style);*/
 
 	child = xml_search_child (tree, "Sheets");
 	if (child == NULL)
@@ -2102,8 +2131,14 @@ xml_probe (const char *filename)
 	xmlNsPtr gmr;
 
 	res = xmlParseFile (filename);
-	if (res == NULL)
+	if (res == NULL) {
+		/* FIXME: make probing silent.  */
+		fprintf (stderr,
+			 "File `%s' does not look like an xml-document\n"
+			 "Please ignore complaints about `Extra content at the end of the document'\n",
+			 filename);
 		return FALSE;
+	}
 
 	if (res->root == NULL) {
 		xmlFreeDoc (res);
@@ -2164,7 +2199,6 @@ gnumeric_xml_read_sheet (const char *filename)
 	
 	xmlFreeDoc (res);
 
-	sheet->modified = FALSE;
 	return sheet;
 }
 
@@ -2204,7 +2238,6 @@ gnumeric_xml_write_sheet (Sheet *sheet, const char *filename)
 	xmlSetDocCompressMode (xml, 9);
 	ret = xmlSaveFile (filename, xml);
 	xmlFreeDoc (xml);
-	sheet->modified = FALSE;
 	if (ret < 0)
 		return -1;
 	return 0;
@@ -2302,7 +2335,7 @@ gnumeric_xml_write_workbook (Workbook *wb, const char *filename)
 void
 xml_init (void)
 {
-	char *desc = _("Old Gnumeric XML file format");
+	char *desc = _("Gnumeric XML file format");
 	
 	file_format_register_open (50, desc, xml_probe, gnumeric_xml_read_workbook);
 	file_format_register_save (".gnumeric", desc, gnumeric_xml_write_workbook);
